/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * QML to DSRT compiler                                                    *
 *                                                                         *
 *     Code developed using JFlex and BYacc/J. To be used on a QoS	   *
 * negotiator mobile agent.                                                *
 *                                                                         *
 * Author:								   *
 * Roberto Speicys Cardoso <speicys@ime.usp.br>				   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

%%

%byaccj
%unicode
%line
%column

%{
  private Parser yyparser;

  public Yylex(java.io.Reader r, Parser yyparser) {
    this(r);
    this.yyparser = yyparser;
  }
%}

/*DOUBLE = [0-9]+ ("." [0-9]+)?*/
DOUBLE = [0-9]+ "." [0-9]+
LONG = [0-9]+
NL  = \n | \r | \r\n
CHAR = [A-Za-z]
STRING = ({LONG}|{DOUBLE}|{CHAR}|\_)+
COMMENT_2 = "//".*{NL}
COMMENT = {COMMENT_2}
%%

/* operators */
";" |
"," |
"." |
":" |
"=" | 
">" | 
"<" | 
"{" | 
"}"    { return (int) yycharat(0); }

/* keywords */

"contract"               { return Parser.CONTRACT; }
"decreasing"		 { return Parser.DECREASING; }
"for"                    { return Parser.FOR; }
"from"			 { return Parser.FROM; }
"increasing"		 { return Parser.INCREASING; }
"numeric"                { return Parser.NUMERIC; }
"percentile"		 { return Parser.PERCENTILE; }
"profile"		 { return Parser.PROFILE; }
"require"		 { return Parser.REQUIRE; }
"type"                   { return Parser.TYPE; }


/* newline */
{NL}   { }

/* float */
{DOUBLE}  { yyparser.yylval = new ParserVal(Double.parseDouble(yytext()));
         return Parser.DOUBLE; }
         
{LONG}  { yyparser.yylval = new ParserVal(Integer.parseInt(yytext()));
         return Parser.LONG; }

{STRING} { yyparser.yylval = new ParserVal(yytext());
           //System.out.println("Teste:"+yyparser.yylval.sval);
	   return Parser.STRING; }

/* whitespace */
[ \t]+ { }

/* Comment */
{COMMENT} { yyparser.yylval = new ParserVal(yytext().substring(0,yylength()-1));
            /*System.out.println("[COMENTARIO] "+yyparser.yylval.sval);*/ }

\b     { System.err.println("Sorry, backspace doesn't work"); }

/* error fallback */
[^]    { System.err.println("Error: unexpected character '"+yytext()+"'"); return -1; }


