/*
 * Created on Oct 15, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package CORBAbroker;

import Pid;
import junit.framework.TestCase;

import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;

import CORBAbroker.BrokerPackage.TicketNotFound;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class TestBroker extends TestCase
{
	private ORB orb;
	Broker bk;
	Pid pid;
	String reservePossible[];
	String reserveImpossible[];
	
	/**
	 * Constructor for TestBroker.
	 * @param arg0
	 */
	public TestBroker(String arg0)
	{
		super(arg0);
	}

	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
		String args[] = null;
		orb  = org.omg.CORBA.ORB.init(args, null);
						
		try
		{
			NamingContextExt nc = NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));		    
			bk =BrokerHelper.narrow(nc.resolve(nc.to_name("DSRT_Broker"))); 
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		}
		
		reservePossible = new String[5];
		reservePossible[0] = new String("CPU_RT_PCPT");
		reservePossible[1] = new String("50");
		reservePossible[2] = new String("34");
		reservePossible[3] = new String();
		reservePossible[4] = new String();
		
		reserveImpossible = new String[5];
		reserveImpossible[0] = new String("CPU_RT_PCPT");
		reserveImpossible[1] = new String("50");
		reserveImpossible[2] = new String("50");
		reserveImpossible[3] = new String();
		reserveImpossible[4] = new String();
								
		pid=new Pid();
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
		
	}
	
	public void testSetUp()
	{
		assertNotNull("bk is null", bk);
	}
	
	public void testReservationSuccessful()
	{
		long tktID = 0;
		
		tktID = bk.reserve(reservePossible, (short)pid.getPid());
		assertTrue(tktID!=-1);
		try
		{
			bk.free(tktID);
		} catch (TicketNotFound e)
		{
			e.printStackTrace();
		}
	}
	
	public void testReservationUnsuccessful()
	{
		long tktID = 0;
		
		tktID = bk.reserve(reserveImpossible, (short)pid.getPid());
		assertTrue(tktID==-1);
	}
	
	public void testReservationLiberation()
	{
		long tktID1 = 0;
		
		tktID1 = bk.reserve(reservePossible, (short)pid.getPid());
		assertTrue(tktID1!=-1);
		try
		{
			bk.free(tktID1);
		} catch (TicketNotFound e)
		{
			e.printStackTrace();
		}
		
		long tktID2 = 0;
		
		tktID2 = bk.reserve(reservePossible, (short)pid.getPid());
		assertTrue(tktID2!=-1);
		try
		{
			bk.free(tktID2);
		} catch (TicketNotFound e1)
		{
			e1.printStackTrace();
		}
	}
	
	public void testDoubleReservation()
	{
		long tktID1 = 0;
		long tktID2 = 0;
		
		tktID1 = bk.reserve(reservePossible, (short)pid.getPid());
		assertTrue(tktID1!=-1);
		
		tktID2 = bk.reserve(reservePossible, (short)pid.getPid());
		assertTrue(tktID2==-1);
		try
		{
			bk.free(tktID1);
		} catch (TicketNotFound e)
		{
			e.printStackTrace();
		}
	}

	public void testDoubleReservationWithLeaseExpiration()
	{
		// Make reservation
		long tktID1 = 0;
		
		tktID1 = bk.reserve(reservePossible, (short)pid.getPid());
		assertTrue(tktID1!=-1);
				
		// Waits until ticket1 times out
		long time=System.currentTimeMillis();
		while(System.currentTimeMillis()<time+7000);
		
		// Make another reservation
		long tktID2 = 0;
		tktID2 = bk.reserve(reservePossible, (short)pid.getPid());
		assertTrue(tktID2!=-1);
		
		try
		{
			bk.free(tktID2);
		} catch (TicketNotFound e)
		{
			e.printStackTrace();
		}
	}
	
	public void testReservationSuggestion()
	{
		long tktID = 0;
		
		tktID = bk.reserve(reserveImpossible, (short)pid.getPid());
		assertTrue(tktID==-1);
		
		if(tktID==-1)
		{
			long ini=System.currentTimeMillis();
			String suggestion[]=bk.suggest(reserveImpossible);
			System.out.println("Time spent in suggestion: "+(System.currentTimeMillis()-ini));
			assertNotNull("suggestion is null", suggestion);
			tktID = bk.reserve(suggestion, (short)pid.getPid());
		}
		
		assertTrue(tktID!=-1);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#finalize()
	 */
	
	protected void finalize() throws Throwable
	{
		//bk.exit();
		super.finalize();
	}
}
