/*
 * Created on Oct 15, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package CORBAbroker;

import java.util.Hashtable;

import cpuJApi.CpuReserve;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class TicketBuilder
{
	/**
	 * Array of createdTickets already created but not used
	 */
	private Hashtable createdTickets;
			
	/**
	 * 	Time (in ms) in which a ticket is valid. Default is 5000
	 */
	private long lease;
	private final long LEASE_DEF=5000;
	/**
	 * Reference to the broker, used for notification of ticket's lease expiration
	 */
	private brokerImpl br;
	
	/**
	 *	Creates a TicketBuilder with default parameters
	 */
	public TicketBuilder(brokerImpl br)
	{
			createdTickets = new Hashtable();
			lease=LEASE_DEF;
			this.br=br;
	}
	
	/**
	 * 	Creates a TicketBuilder for createdTickets with custom validation time
	 * 
	 * @param lease Validation time of the ticket
	 */
	public TicketBuilder(brokerImpl br, long lease)
	{
		createdTickets=new Hashtable();
		this.lease=lease;	
		this.br=br;
	}
	
	/**
	 * 	Creates a new ticket
	 * 
	 * @param procIndex Index of the processor reserved
	 * @param cr Cpu Reserve reserved
	 * @return Ticket ID
	 */
	public Long create(int pid, CpuReserve cr)
	{
		Ticket tkt = new Ticket(pid, cr, this.lease, this);
		createdTickets.put(tkt.getId(), tkt);
		
		return(tkt.getId());
	}
	
	public void leaseExpired(Long ticketId)
	{
		br.leaseExpired((Ticket)createdTickets.get(ticketId));		
		createdTickets.remove(ticketId);
	}
	
	public boolean isCreated(Long ticketId)
	{
		return(createdTickets.containsKey(ticketId));
	}
	
	public void start(Long ticketId)
	{
		Ticket used=(Ticket)createdTickets.get(ticketId);
		used.cancelTimeout();
		createdTickets.remove(ticketId);
	}
	
	public Ticket getCreated(Long ticketId)
	{
		return((Ticket)createdTickets.get(ticketId));
	}
}
