/*
 * Created on Nov 12, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package CORBAbroker;

import java.net.InetAddress;

import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.CosNaming.NamingContextPackage.AlreadyBound;

import CORBAbroker.BrokerPackage.TicketNotFound;
import cpuJApi.CpuJApi;
import cpuJApi.CpuReserve;
import cpuJApi.SchedStats;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class brokerImpl extends BrokerPOA
{
	TicketBuilder tb;
	
	public final long INVALID_TICKET=-1;
	private final int MAX_PARAM_RESERVATION=5;
	
	static boolean exit;
		
	public brokerImpl()
	{
		CpuJApi.init();
		
		// SchedStats ss = new SchedStats();
		
		exit=false;
		
		/*if(CpuJApi.getSchedStats(ss))
		{		
			// Creates the ticket builder which will be responsible for create all the reservation tickets
			*/
			tb = new TicketBuilder(this);
			
			// Loads broker variables with real values
//			numProc = ss.numProcessor;
//			this.processors = ss.processors;
//			System.out.println(numProc);
//			for(int i=0; i<numProc; i++)
//				System.out.println("\tProcessor "+numProc+" has "+processors[i].availPermUtil+" available for real time processes");
//		}
//		else
//			System.out.println("Unable to get scheduled statistics");		
	}
	
	synchronized public void leaseExpired(Ticket tk)
	{
		try
		{
			free(tk.getId().longValue());
		} catch (TicketNotFound e)
		{
			e.printStackTrace();
		}
	}
	
	private double getUtil(CpuReserve cr)
	{
		switch (cr.serviceClass)	
		{	
			case CpuReserve.CPU_RT_PCPT:
			case CpuReserve.CPU_RT_PCCPT:
				return((double)cr.peakProcessingTime / (double)cr.period);
			case CpuReserve.CPU_RT_PVPT:
				return((double)cr.sustainableProcessingTime / (double)cr.period);
			case CpuReserve.CPU_RT_ACPU:
				return(cr.peakProcessingUtil);
			default:
				System.out.println("getUtil: unknown class\n");
		  return(0.0);
		}
	}
	
	public static void main(String[] args)
	{	
		org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
		org.omg.PortableServer.POA poa;
		
		System.out.println(System.getProperty("java.library.path"));
		
		try
		{
			poa =
				org.omg.PortableServer.POAHelper.narrow(
					orb.resolve_initial_references("RootPOA"));
					
			poa.the_POAManager().activate();
			
			org.omg.CORBA.Object o;	
			o = poa.servant_to_reference(new brokerImpl());
			
			org.omg.CosNaming.NamingContextExt nc =
				org.omg.CosNaming.NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));

			String hostname = InetAddress.getLocalHost().getHostName();
			NamingContextExt host=null;
					
			try
			{
				host = NamingContextExtHelper.narrow(nc.bind_new_context(nc.to_name("DSRT_Broker")));
			}
			catch(AlreadyBound e)
			{
				host = NamingContextExtHelper.narrow(nc.resolve(nc.to_name("DSRT_Broker")));
			}
			
			try
			{
				host.bind(nc.to_name(hostname), o);
			}
			catch(AlreadyBound e)
			{
				host.unbind(nc.to_name(hostname));
				host.bind(nc.to_name(hostname), o);
			}				
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}		
		orb.run();
	}

	/* (non-Javadoc)
	 * @see CORBAbroker.BrokerOperations#reserve(short, CORBAbroker.BrokerPackage.CpuReserve)
	 */
	public long reserve(String cr[], short pid)
	{
		CpuReserve dsrtCr = new CpuReserve();
		dsrtCr.parse(cr, 0);
		Long ticketID = new Long(INVALID_TICKET);
		
		if(CpuJApi.reserve(dsrtCr, pid))
		{
			//	Creates a new ticket for the reservation requested
			 ticketID=tb.create(pid, dsrtCr);
			 System.out.println("Reservation Successful");
		}
		
		else
			System.out.println("Reservation Unsuccessful");
						
		return ticketID.longValue();
	}
	
	public String [] suggest(String cr[])
	{
		CpuReserve dsrtCr = new CpuReserve();
		dsrtCr.parse(cr, 0);
		
		String suggestion[] = null;
		
		switch (dsrtCr.serviceClass)
		{	
			case CpuReserve.CPU_RT_PCPT:
			case CpuReserve.CPU_RT_PCCPT:
				suggestion = suggestReservation(dsrtCr, 0, dsrtCr.peakProcessingTime);
				break;
			
			case CpuReserve.CPU_RT_PVPT:
				suggestion = suggestReservation(dsrtCr, 0, dsrtCr.sustainableProcessingTime);
				break;
			
			case CpuReserve.CPU_RT_ACPU:
				suggestion = suggestReservation(dsrtCr, 0, (long)dsrtCr.peakProcessingUtil);
				break;
		}
		
// Second method to estimate ideal reservation 		
//		SchedStats ss=new SchedStats(); 
//		CpuJApi.getSchedStats(ss);
//		suggestion=suggestReservation(dsrtCr, ss);

		System.out.println(suggestion[0]+" "+suggestion[1]+" "+suggestion[2]);
					
		if(suggestion==null)
		{
			// Unable to find a suggestion
			System.out.println("Error finding suggestion");
			return(null);
		}
		
		return(suggestion);
	}
	
	private String[] suggestReservation(CpuReserve cr, long min, long max)
	{
		// Essentially a binary search for an ideal reservation value
				
		// K = Ki
		if(max-min==1)
		{
			System.out.println("Found ideal value of "+min);
			String suggestion[];
			
			switch (cr.serviceClass)
			{	
				case CpuReserve.CPU_RT_PCPT:
				case CpuReserve.CPU_RT_PCCPT:
					cr.peakProcessingTime=min;
					suggestion=parse(cr.toString(), 3);
					return(suggestion);
				
				case CpuReserve.CPU_RT_PVPT:
					cr.sustainableProcessingTime=min;
					suggestion=parse(cr.toString(), 5);
					return(suggestion);
				
				case CpuReserve.CPU_RT_ACPU:
					cr.peakProcessingUtil=min;
					suggestion=parse(cr.toString(), 2);	
					return(suggestion);
			}
		}
		
		else
		{
			long tryValue = (long)(max+min)/2;
			System.out.println("Trying to reserve value "+tryValue);
			
			switch (cr.serviceClass)
			{	
				case CpuReserve.CPU_RT_PCPT:
				case CpuReserve.CPU_RT_PCCPT:
					cr.peakProcessingTime=tryValue;
					break;
									
				case CpuReserve.CPU_RT_PVPT:
					cr.sustainableProcessingTime=tryValue;
					break;
			
				case CpuReserve.CPU_RT_ACPU:
					cr.peakProcessingUtil=tryValue;
					break;
			}
			
			// K > Ki			
			if(CpuJApi.reserve(cr))
			{
				CpuJApi.free();
				System.out.println("Can find a higher value than "+tryValue);
				return(suggestReservation(cr, tryValue, max));
			}
			
			// K < Ki
			else
			{
				System.out.println("Must find a lower value than "+tryValue);
				return(suggestReservation(cr, min, tryValue));
			}
		}
		
		return null;
	}
	
	private String [] suggestReservation(CpuReserve cr, SchedStats ss)
	{
		// Finds the processor with greatest idle period
		double available=0.0;
		for(int i=0; i<ss.numProcessor; i++)
		{
			if(ss.processors[i].availPermUtil>available)
				available=ss.processors[i].availPermUtil;
		}
		
		if(available==0.0)
			return(null);
		
		long newValue;
		String suggestion[];
		
		switch (cr.serviceClass)
		{	
			case CpuReserve.CPU_RT_PCPT:
			case CpuReserve.CPU_RT_PCCPT:
				newValue = (long)((double)cr.period*available);
				cr.peakProcessingTime=newValue;
				suggestion=parse(cr.toString(), 3);
				return(suggestion);
				
			case CpuReserve.CPU_RT_PVPT:
				newValue = (long)((double)cr.period*available);
				cr.sustainableProcessingTime=newValue;
				suggestion=parse(cr.toString(), 5);
				return(suggestion);
				
			case CpuReserve.CPU_RT_ACPU:
				cr.peakProcessingUtil=available;
				suggestion=parse(cr.toString(), 2);	
				return(suggestion);
		}
		
		return(null);
	}
	
	
	
	/**
	 * @param string
	 * @param i
	 * @return
	 */
	private String [] parse(String cpuReserve, int numArgs)
	{
		String leftover;
		String args[]=new String[this.MAX_PARAM_RESERVATION];
		
		leftover=new String(cpuReserve);
		
		for(int i=0; i<numArgs; i++)
		{
			int index = leftover.indexOf('/');
			args[i] = leftover.substring(0,index);
			leftover=leftover.substring(index+1);
		}
		
		for(int i=numArgs; i<this.MAX_PARAM_RESERVATION; i++)
		{
			args[i] = new String();
		}
		
		args[0]=new String("CPU_").concat(args[0]);
		
		return args;
	}

	public void free(long ticketId) throws TicketNotFound
	{
		if(!tb.isCreated(new Long(ticketId)))
			throw new TicketNotFound();
		
		Ticket tk = tb.getCreated(new Long(ticketId));
		CpuJApi.free(tk.getPid());		
	}

	/* (non-Javadoc)
	 * @see CORBAbroker.BrokerOperations#startReservation(long)
	 */
	public void start(long ticketId) throws TicketNotFound
	{
		//		Checks if ticket was really created
		if(!tb.isCreated(new Long(ticketId)))
			throw new TicketNotFound();
			
		else
		{
			Ticket reserve=tb.getCreated(new Long(ticketId));
			tb.start(new Long(ticketId));
		 	CpuJApi.start(reserve.getPid());
		 }	
	}

	/* (non-Javadoc)
	 * @see CORBAbroker.BrokerOperations#stopReservation(long)
	 */
	public void stop(long ticketId) throws TicketNotFound
	{
		if(!tb.isCreated(new Long(ticketId)))
			throw new TicketNotFound();
			
		else
		{
			Ticket reserve=tb.getCreated(new Long(ticketId));
			CpuJApi.stop(reserve.getPid());
		}
	}
}