/*
 * Created on Oct 13, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package broker;

import cpuJApi.CpuJApi;
import cpuJApi.CpuReserve;
import cpuJApi.Processor;
import cpuJApi.SchedStats;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class Broker extends Thread
{
	int numProc;
	Processor [] processors;
	TicketBuilder tb;
	
	public final long INVALID_TICKET=-1;
	static boolean exit;

	public Broker()
	{
		CpuJApi.init();
		SchedStats ss = new SchedStats();
		exit=false;
		
		if(CpuJApi.getSchedStats(ss))
		{		
			// Creates the ticket builder which will be responsible for create all the reservation tickets
			tb = new TicketBuilder(this);
			
			// Loads broker variables with real values
			numProc = ss.numProcessor;
			this.processors = ss.processors;
			System.out.println(numProc);
			for(int i=0; i<numProc; i++)
				System.out.println("\tProcessor "+numProc+" has "+processors[i].availPermUtil+" available for real time processes");
		}
		else
			System.out.println("Unable to get scheduled statistics");
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public static void main(String args[])
	{
		Broker br=new Broker();
		br.start();		
	}
	
	public void run()
	{
		while(!exit)
		{
			try
			{
				sleep(1000);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}
	}
	
	public long reserve(int pid, CpuReserve cr)
	{
		double utilization = getUtil(cr);
		boolean processorAvail = false;
		int processorIndex=-1;
		Long ticketID = new Long(INVALID_TICKET);
		
		// Try to fit the reservation on any of the available processors
		for(int i=0; i<numProc; i++)
		{
			if(processors[i].availPermUtil>=utilization)
			{
				processorAvail = true;
				processorIndex=i;
				break;
			}	
		}
		
		if(processorAvail)
		{
			// Creates a new ticket for the reservation requested
			ticketID=tb.create(pid, processorIndex, cr);
			System.out.println("Available utilization before "+processors[processorIndex].availPermUtil);
			processors[processorIndex].availPermUtil-=utilization;
			System.out.println("Available utilization after "+processors[processorIndex].availPermUtil);
		}
		else
		{
			// Gera uma nova reserva de Cpu que possa ser atendida
			
		}
		
		return ticketID.longValue();
	}
	
	private double getUtil(CpuReserve cr)
	{
	  	switch (cr.serviceClass)	
		{	
			case CpuReserve.CPU_RT_PCPT:
			case CpuReserve.CPU_RT_PCCPT:
		  		return((double)cr.peakProcessingTime / (double)cr.period);
			case CpuReserve.CPU_RT_PVPT:
		  		return((double)cr.sustainableProcessingTime / (double)cr.period);
			case CpuReserve.CPU_RT_ACPU:
		  		return(cr.peakProcessingUtil);
			default:
		  		System.out.println("getUtil: unknown class\n");
		  return(0.0);
		}
	}
	
	synchronized public void leaseExpired(Ticket tk)
	{
		double utilization=getUtil(tk.getCr());
		
		processors[tk.getProcIndex()].availPermUtil+=utilization;
		System.out.println("Lease expired for ticket "+tk.getId());
	}
	
	public boolean makeReservation(long ticketId)
	{
		if(!tb.isCreated(new Long(ticketId)))
			return(false);
		
		Ticket reserve = tb.getCreated(new Long(ticketId));
		tb.reserve(new Long(ticketId));
		CpuJApi.reserve(reserve.getCr(), reserve.getPid());
		
		return(true);
	}
	
	public boolean startReservation(long ticketId)
	{
		// Checks if ticket was really reserved
		if(tb.isReserved(new Long(ticketId)))
		{
			Ticket reserve=tb.getUsed(new Long(ticketId));
			CpuJApi.start(reserve.getPid());
			return(true);
		}	
		return(false);
	}
	
	public boolean stopReservation(long ticketId)
	{
		if(tb.isReserved(new Long(ticketId)))
		{
			Ticket reserve=tb.getUsed(new Long(ticketId));
			CpuJApi.stop(reserve.getPid());
			return(true);
		}
		return(false);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#finalize()
	 */
	protected void finalize() throws Throwable
	{
		CpuJApi.destroy();
		super.finalize();
	}
}
