package broker;

/* 
 * Released Date: March 7 99
 * This is the Dynamic Soft CPU Scheduler II release by the 
 *   Monet Group, 
 *   http://cairo.cs.uiuc.edu/
 *   Department of Computer Science, 
 *   University of Illinois at Urbana Champaign.  
 * 
 * This release has been tested under SUN OS 5.6.
 *
 * Contacting Authors:
 *  Hao-hua Chu (h-chu3@cs.uiuc.edu)
 *  Klara Nahrstedt (klara@cs.uiuc.edu)
 */

import java.applet.Applet;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import cpuJApi.AdaptStrategy;
import cpuJApi.CpuReserve;
import cpuJApi.Double3;
import cpuJApi.Proc;
import cpuJApi.CpuJApi;
import cpuJApi.Processor;
import cpuJApi.SchedStats;

public class CpuMonitor extends Applet implements Runnable, ActionListener {
    ServerStats serverStats;
    ClientStats clientStats;
    Button quit;
    private Thread updateThread = null;
    public static int N = 10;
    public static int P = 2000;
    public static int IASIZE = 20;
    public static int FASIZE = 10;
    
    public void init(Frame f) {
	serverStats = new ServerStats();
	clientStats = new ClientStats(f);
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gridbag);
    
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.BOTH;
	Label ss = new Label("Scheduler Information");
	/*
	Font font = getFont();
	Font newFont = new Font(font.getName(), Font.BOLD, font.getSize()+1);
	ss.setFont(newFont);
	*/
	gridbag.setConstraints(ss, c);
	add(ss);
	gridbag.setConstraints(serverStats, c);
	add(serverStats);

	Label ps = new Label("Process Information");
	// ps.setFont(newFont);
	gridbag.setConstraints(ps, c);
	add(ps);

	gridbag.setConstraints(clientStats, c);
	add(clientStats);
	
	quit = new Button("Quit");
	gridbag.setConstraints(quit, c);
	quit.addActionListener(this);
	add(quit);
    }

    public Insets getInsets() {
	return new Insets(5,5,5,5);
    }
  
    public void start() {
	CpuJApi.init();
	if (updateThread == null) {
	    updateThread = new Thread(this);
	    updateThread.start();
	}
    }

    public void run() {
	Thread currentThread = Thread.currentThread();
	while (updateThread == currentThread) {
	    updateState();
	    repaint();
	    try {
		Thread.sleep(P);
	    } catch(InterruptedException e) {
		break;
	    }
	}
    }

    public void updateState() {
	serverStats.updateState();
	clientStats.updateState();
    }
    
    public void actionPerformed(ActionEvent event) {
	Object target = event.getSource();
	if (target == quit) {
	    CpuJApi.destroy();
	    System.exit(0);
	}
    }

    public static String doubleToString(double f) {
	return(Double.toString(Math.round(f*1000.0)/1000.0));
    }

    /** Executed only when this program runs as an application. */
    public static void main(String[] args) {
	//Create a new window.
	Frame f = new Frame("CpuMonitor Applet/Application");
	f.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		System.exit(0);
	    }
	});
	
	CpuMonitor cpuMonitor = new CpuMonitor();
	//Initialize the Converter instance.
	cpuMonitor.init(f);
	cpuMonitor.start();
	f.add("Center", cpuMonitor);
	f.pack();        //Resizes the window to its natural size.
	f.setVisible(true);
    }
}


class RecPanel extends Panel {
    public void paint(Graphics g) {
        Dimension d = getSize();
        g.drawRect(0,0, d.width - 1, d.height - 1);
    }
    public Insets getInsets() {
        return new Insets(5,5,5,5);
    }
}


class ServerStats extends RecPanel {

    // Prid rtPart rtAvail rtUsage ovPart ovUsage/Burst/Nonconform/Probe tsPart  

    TextField pridTab[];
    TextField rtPartTab[];
    TextField rtAvailTab[];
    TextField rtUsageTab[];
    TextField ovPartTab[];
    TextField ovUsageTab[];
    TextField tsPartTab[];

    SchedStats _ss;

    ServerStats() {
	int i;

	Panel pridColumn = new Panel();
	Panel rtPartColumn = new Panel();
	Panel rtAvailColumn = new Panel();
	Panel rtUsageColumn = new Panel();
	Panel ovPartColumn = new Panel();
	Panel ovUsageColumn = new Panel();
	Panel tsPartColumn = new Panel();

	pridColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESSOR+1,1));
	rtPartColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESSOR+1,1));
	rtAvailColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESSOR+1,1));
	rtUsageColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESSOR+1,1));
	ovPartColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESSOR+1,1));
	ovUsageColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESSOR+1,1));
	tsPartColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESSOR+1,1));

	pridColumn.add(new Label("Processor ID"));
	rtPartColumn.add(new Label("rtPart"));
	rtAvailColumn.add(new Label("rtAvail"));
	rtUsageColumn.add(new Label("rtUsage"));
	ovPartColumn.add(new Label("ovPart"));
	ovUsageColumn.add(new Label("ovUsage/Burst/Nonconform/Probe"));
	tsPartColumn.add(new Label("tsPart"));

	pridTab = new TextField[CpuJApi.MAX_PROCESSOR];
	rtPartTab = new TextField[CpuJApi.MAX_PROCESSOR];
	rtAvailTab = new TextField[CpuJApi.MAX_PROCESSOR];
	rtUsageTab = new TextField[CpuJApi.MAX_PROCESSOR];
	ovPartTab = new TextField[CpuJApi.MAX_PROCESSOR];
	ovUsageTab = new TextField[CpuJApi.MAX_PROCESSOR];
	tsPartTab = new TextField[CpuJApi.MAX_PROCESSOR];

	for (i=0; i<CpuJApi.MAX_PROCESSOR; i++) {
	    pridTab[i] = new TextField();
	    rtPartTab[i] = new TextField();
	    rtAvailTab[i] = new TextField();
	    rtUsageTab[i] = new TextField();
	    ovPartTab[i] = new TextField();
	    ovUsageTab[i] = new TextField();
	    tsPartTab[i] = new TextField();

	    pridTab[i].setEditable(false);
	    rtPartTab[i].setEditable(false);
	    rtAvailTab[i].setEditable(false);
	    rtUsageTab[i].setEditable(false);
	    ovPartTab[i].setEditable(false);
	    ovUsageTab[i].setEditable(false);
	    tsPartTab[i].setEditable(false);

	    pridColumn.add(pridTab[i]);
	    rtPartColumn.add(rtPartTab[i]);
	    rtAvailColumn.add(rtAvailTab[i]);
	    rtUsageColumn.add(rtUsageTab[i]);
	    ovPartColumn.add(ovPartTab[i]);
	    ovUsageColumn.add(ovUsageTab[i]);
	    tsPartColumn.add(tsPartTab[i]);
	}

	setLayout(new FlowLayout());
	add(pridColumn);
	add(rtPartColumn);
	add(rtAvailColumn);
	add(rtUsageColumn);
	add(ovPartColumn);
	add(ovUsageColumn);
	add(tsPartColumn);
	_ss = null;
    }

    void updateState() {
	SchedStats ss = new SchedStats();
	Processor pr;
	if (CpuJApi.getSchedStats(ss)) {

	    for (int i=0; i<ss.numProcessor; i++) {
		pr = ss.processors[i];
		pridTab[i].setText(Integer.toString(i));
		rtPartTab[i].setText(CpuMonitor.doubleToString(pr.permRatio));
		rtAvailTab[i].setText(CpuMonitor.doubleToString(pr.availPermUtil));
		ovPartTab[i].setText(CpuMonitor.doubleToString(pr.overrunRatio));			
		tsPartTab[i].setText(CpuMonitor.doubleToString(pr.tsRatio));		
	    }		    
	    pridTab[ss.numProcessor].setText("Total");
	    rtPartTab[ss.numProcessor].setText(CpuMonitor.doubleToString(ss.permRatio));
	    rtAvailTab[ss.numProcessor].setText(CpuMonitor.doubleToString(ss.availPermUtil()));
	    ovPartTab[ss.numProcessor].setText(CpuMonitor.doubleToString(ss.overrunRatio));
	    tsPartTab[ss.numProcessor].setText(CpuMonitor.doubleToString(ss.tsRatio));

	    if (_ss != null) {
		long t = ss.clock - _ss.clock;
		if (t > 0) {
		    double allRtUsage = 0;
		    double allOvUsage = 0;
		    double allOvBurst = 0;
		    double allOvNonconform = 0;
		    double allOvProbe = 0;
		    for (int i=0; i<ss.numProcessor; i++) {
			pr = ss.processors[i];
			Processor _pr = _ss.processors[i];
			
			double rtUsage = (double)(pr.permUsage - _pr.permUsage) / (double)t;
			double ovBurst = (double)(pr.burstUsage - _pr.burstUsage) / (double)t;
			double ovNonconform = (double)(pr.nonconformUsage - _pr.nonconformUsage) / (double)t;
			double ovProbe = (double)(pr.probeUsage - _pr.probeUsage) / (double)t;
			double ovUsage = ovBurst + ovNonconform + ovProbe; 

			rtUsageTab[i].setText(CpuMonitor.doubleToString(rtUsage));
			ovUsageTab[i].setText(CpuMonitor.doubleToString(ovUsage) + "/" +
					      CpuMonitor.doubleToString(ovBurst) + "/" +
					      CpuMonitor.doubleToString(ovNonconform) + "/" +
					      CpuMonitor.doubleToString(ovProbe));
			allRtUsage += rtUsage;
			allOvUsage += ovUsage;
			allOvBurst += ovBurst;
			allOvNonconform += ovNonconform;
			allOvProbe += ovProbe;
		    }
		    rtUsageTab[ss.numProcessor].setText(CpuMonitor.doubleToString(allRtUsage));
		    ovUsageTab[ss.numProcessor].setText(CpuMonitor.doubleToString(allOvUsage) + "/" +
							CpuMonitor.doubleToString(allOvBurst) + "/" +
							CpuMonitor.doubleToString(allOvNonconform) + "/" +
							CpuMonitor.doubleToString(allOvProbe));
		}
	    }
	}
	_ss = ss;
    }
}



class Client implements ActionListener {
    TextField _pid;
    TextField _prid;
    TextField _state;
    TextField _reserve;    
    TextField _adapt;
    Button _modify;
    TextField _usage;
    TextField _status;
    ModifyDialog _rd;
    Frame _parent;
    Proc _p;

    Client(Frame parent, Panel pidColumn, Panel pridColumn, Panel stateColumn, 
	   Panel reserveColumn, Panel adaptColumn, 
	   Panel modifyUsageColumn, Panel statusColumn) {
	_parent = parent;
	_pid = new TextField(); 
	_pid.setEditable(false);
	pidColumn.add(_pid);
	_prid = new TextField(); 
	_prid.setEditable(false);
	pridColumn.add(_prid);
	_state = new TextField();
	_state.setEditable(false);
	stateColumn.add(_state);
	_reserve = new TextField();
	_reserve.setEditable(false);
	reserveColumn.add(_reserve);
	_adapt = new TextField();
	_adapt.setEditable(false);
	adaptColumn.add(_adapt);
	_modify = new Button("Modify");
	_modify.addActionListener(this);
	modifyUsageColumn.add(_modify);
	_usage = new TextField();
	_usage.setEditable(false);
	modifyUsageColumn.add(_usage);
	_status = new TextField(20);
	_status.setEditable(false);
	statusColumn.add(_status);
	_rd = null;
	init();
    }

    void init() {
	_pid.setText("-");
	_prid.setText("-");
	_state.setText("-");
	_reserve.setText("-");
	_adapt.setText("-");
	_usage.setText("-");
	_status.setText("-");
	if (_rd != null)
	    _rd.setVisible(false);
	_rd = null;
	_p = null;
    }

    void update(Proc p) {
	_pid.setText(Integer.toString(p.pid));
	_prid.setText(Integer.toString(p.prid));
	_state.setText(p.stateToString());
	if (p.state == Proc.PSTATE_INIT ||
	    p.state == Proc.PSTATE_PROBE_INIT ||
	    p.state == Proc.PSTATE_PROBE) {
	    _reserve.setText("-");
	    _adapt.setText("-");
	    _p = p;
	    return;
	}

	_reserve.setText(p.cpuReserve.toString());
	_adapt.setText(p.adaptStrategy.toString());

	if (_p == null) {
	    _usage.setText("-");
	    _status.setText("-");
	    _p = p;
	    return;
	}
	
	long t = p.ps.clock - _p.ps.clock;
	int n = p.ps.accItr - _p.ps.accItr;
	if (t == 0 || n == 0) {
	    _usage.setText("-");
	    _status.setText("-");
	    _p = p;
	    return;
	}

	_usage.setText(Double3.toString((double)(p.ps.accUsage - _p.ps.accUsage)/(double)t));
	_status.setText(Double3.toString((double)(p.ps.accNumOverrun - _p.ps.accNumOverrun)/(double)n) + "/" +
			Double3.toString((double)(p.ps.accNumNonconform - _p.ps.accNumNonconform)/(double)n) + "/" +
			Double3.toString((double)(p.ps.accNumPermNonconform - _p.ps.accNumPermNonconform)/(double)n) + "/" +
			Double3.toString((double)(p.ps.accNumDeadlineMiss - _p.ps.accNumDeadlineMiss)/(double)n));
	_p = p;
    }
    
    public void actionPerformed(ActionEvent ae) {
	Object target = ae.getSource();
	if (target == _modify) {
	    if (_p != null) {
		_rd = new ModifyDialog(_parent, _p);
		_rd.setVisible(true);
	    }
	}
    }
}


class ClientStats extends RecPanel {
    Client[] _clientTab;
    Frame _parent;

    ClientStats(Frame parent) {
	_parent = parent;
	Panel pidColumn = new Panel();
	Panel pridColumn = new Panel();
	Panel stateColumn = new Panel();
	Panel reserveColumn = new Panel();
	Panel adaptColumn = new Panel();
	Panel modifyUsageColumn = new Panel();
 	Panel statusColumn = new Panel();

	pidColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESS+2,1));
	pidColumn.add(new Label());
	pidColumn.add(new Label("PID"));

	pridColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESS+2,1));
	pridColumn.add(new Label());
	pridColumn.add(new Label("PRID"));

	stateColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESS+2,1));
	stateColumn.add(new Label());
	stateColumn.add(new Label("State"));
	
	reserveColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESS+2,1));
	reserveColumn.add(new Label("Reservation"));
	reserveColumn.add(new Label("Class/Period/PPT/SPT/BT/PPU"));	
	
	adaptColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESS+2,1));
	adaptColumn.add(new Label("Adaptation"));
	adaptColumn.add(new Label("Strategy/Wsize/Alpha/Max/Min/NcFreq"));	
	

	modifyUsageColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESS+2,2));
	modifyUsageColumn.add(new Label());
	modifyUsageColumn.add(new Label());
	modifyUsageColumn.add(new Label("Modify"));
	modifyUsageColumn.add(new Label("Usage"));

	statusColumn.setLayout(new GridLayout(CpuJApi.MAX_PROCESS+2,1));
	statusColumn.add(new Label("Status"));
	statusColumn.add(new Label("Ov/Nc/Pnc/DMiss"));
	

	_clientTab = new Client[CpuJApi.MAX_PROCESS];	
	for (int i=0; i<CpuJApi.MAX_PROCESS; i++)
	    _clientTab[i] = new Client(_parent, pidColumn, pridColumn, stateColumn, 
				       reserveColumn, adaptColumn, modifyUsageColumn, 
				       statusColumn);

	setLayout(new FlowLayout());
	add(pidColumn);
	add(pridColumn);
	add(stateColumn);
	add(reserveColumn);
	add(adaptColumn);
	add(modifyUsageColumn);
	add(statusColumn);
    }

    void updateState() {
	int pidTab[] = new int[CpuJApi.MAX_PROCESS];
	CpuJApi.getPidTab(pidTab, CpuJApi.MAX_PROCESS);
	
	for (int i=0; i<CpuJApi.MAX_PROCESS; i++) {
	    if (pidTab[i] != 0) {
		Proc p = new Proc();
		if (CpuJApi.getProc(p, pidTab[i]))
		    _clientTab[i].update(p);
		else
		    _clientTab[i].init();
	    } 
	    else 
		_clientTab[i].init();
     	}
    }
}


class ModifyDialog extends Dialog implements ActionListener, ItemListener {
    Button _setReserve, _setAdapt, _setBoth, _close;

    Choice _serviceClassChoice;
    Choice _adaptStrategyChoice;
    TextField _period, _ppt, _spt, _bt, _ppu;    
    TextField _windowSize, _alpha, _max, _min, _nonconformFreq;
    int _pid;
    
    ModifyDialog(Frame parent, Proc p) {
	super(parent, "Modify Reservation", false);
	_pid = p.pid;

	Label pidLabel = new Label("PID " + _pid);

	// reservation class 
	_serviceClassChoice = new Choice();
	_serviceClassChoice.add("Periodic Constant Processing Time");
	_serviceClassChoice.add("Periodic Variable Processing Time");
	_serviceClassChoice.add("Aperiodic Constant Processing Time");
	_serviceClassChoice.add("Event");
	_serviceClassChoice.addItemListener(this);
	
	// reservation parameters
	RecPanel rparamColumn =  new RecPanel();
	rparamColumn.setLayout(new GridLayout(6,2));

	_period = new TextField(10);
	_ppt = new TextField(10);
	_spt = new TextField(10);
	_bt = new TextField(10);
	_ppu = new TextField(10);
	
	rparamColumn.add(new Label("Period:"));
	rparamColumn.add(_period);
	rparamColumn.add(new Label("PeakProcessingTime:"));
	rparamColumn.add(_ppt);
	rparamColumn.add(new Label("SustainableProcessingTime:"));
	rparamColumn.add(_spt);
	rparamColumn.add(new Label("BurstTolerance:"));
	rparamColumn.add(_bt);
	rparamColumn.add(new Label("PeakProcessingUtil:"));
	rparamColumn.add(_ppu);

	// adaptation strategy parameters
	_adaptStrategyChoice = new Choice();
	_adaptStrategyChoice.add("NONE");
	_adaptStrategyChoice.add("Exponential");
	_adaptStrategyChoice.add("Statistical");
	_adaptStrategyChoice.addItemListener(this);

	// adaptation class parameter
	Panel aparamColumn = new RecPanel();
	_windowSize = new TextField(10);
	_alpha = new TextField(10);
	_max =  new TextField(10);
	_min =  new TextField(10);
	_nonconformFreq =  new TextField(10);
	
	aparamColumn.setLayout(new GridLayout(6,2));
	aparamColumn.add(new Label("windowSize"));
	aparamColumn.add(_windowSize);
	aparamColumn.add(new Label("alpha"));
	aparamColumn.add(_alpha);
	aparamColumn.add(new Label("nonconformFreq"));
	aparamColumn.add(_nonconformFreq);	
	aparamColumn.add(new Label("max"));
	aparamColumn.add(_max);
	aparamColumn.add(new Label("min"));
	aparamColumn.add(_min);
	
	_setReserve = new Button("Set Reservation");
	_setAdapt = new Button("Set Adaptation");
	_setBoth = new Button("Set Both");
	_close = new Button("Close");
	_setReserve.addActionListener(this);
	_setAdapt.addActionListener(this);
	_setBoth.addActionListener(this);
	_close.addActionListener(this);

	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gridbag);

	c.fill = GridBagConstraints.BOTH;
	c.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(pidLabel, c);
	add(pidLabel);
	gridbag.setConstraints(_serviceClassChoice, c);
	add(_serviceClassChoice);
	gridbag.setConstraints(rparamColumn, c);
	add(rparamColumn);
	gridbag.setConstraints(_adaptStrategyChoice, c);
	add(_adaptStrategyChoice);
	gridbag.setConstraints(aparamColumn, c);
	add(aparamColumn);
	gridbag.setConstraints(_setReserve, c);
	add(_setReserve);
	gridbag.setConstraints(_setAdapt, c);
	add(_setAdapt);
	gridbag.setConstraints(_setBoth, c);
	add(_setBoth);
	gridbag.setConstraints(_close, c);
	add(_close);
	
	selectServiceClass(p.cpuReserve.serviceClass);
	selectAdaptStrategy(p.adaptStrategy.strategy);
	update(p);
	pack();
    }
    

    void update(Proc p) {
	CpuReserve cr = p.cpuReserve;
	_period.setText(Long.toString(cr.period/1000));
	_ppt.setText(Long.toString(cr.peakProcessingTime/1000));
	_spt.setText(Long.toString(cr.sustainableProcessingTime/1000));
	_bt.setText(Long.toString(cr.burstTolerance/1000));
	_ppu.setText(Double3.toString(cr.peakProcessingUtil));
	selectServiceClass(cr.serviceClass);

	AdaptStrategy as = p.adaptStrategy;
	_windowSize.setText(Integer.toString(as.windowSize));
	_alpha.setText(Double3.toString(as.alpha));
	_max.setText(Double3.toString(as.max/1000));
	_min.setText(Double3.toString(as.min/1000));
	_nonconformFreq.setText(Double3.toString(as.nonconformFreq));
	selectAdaptStrategy(as.strategy);
    }
    
    void selectServiceClass(int serviceClass) {
	_period.setVisible(false);
	_ppt.setVisible(false);
	_spt.setVisible(false);
	_bt.setVisible(false);
	_ppu.setVisible(false); 
	switch(serviceClass) {
	case CpuReserve.CPU_RT_PCPT:
	    _period.setVisible(true);
	    _ppt.setVisible(true);
	    break;
	case CpuReserve.CPU_RT_PVPT:
	    _period.setVisible(true);
	    _ppt.setVisible(true);
	    _spt.setVisible(true);
	    _bt.setVisible(true);	   
	    break;
	case CpuReserve.CPU_RT_ACPU:
	    _ppu.setVisible(true); 
	    break;
	case CpuReserve.CPU_RT_EVENT:
	     _ppt.setVisible(true);
	    break;
	}
	pack();
    }

    void selectAdaptStrategy(int strategy) {	
	_windowSize.setVisible(false);
	_alpha.setVisible(false);
	_max.setVisible(false);
	_min.setVisible(false);
	_nonconformFreq.setVisible(false);
	switch(strategy) {
	case AdaptStrategy.ADAPT_NONE:
	    break;
	case AdaptStrategy.ADAPT_EXPO:
	    _windowSize.setVisible(true);
	    _alpha.setVisible(true);
	    _max.setVisible(true);
	    _min.setVisible(true);
	    break;
	case AdaptStrategy.ADAPT_STAT:
	    _windowSize.setVisible(true);
	    _nonconformFreq.setVisible(true);
	    _max.setVisible(true);
	    _min.setVisible(true);
	    break;
	default:
	    System.out.println("selectAdaptStrategy: Unknown strategy");
	}
	pack();
    }

    public void itemStateChanged(ItemEvent e) {
	selectAdaptStrategy(_adaptStrategyChoice.getSelectedIndex());
	selectServiceClass(_serviceClassChoice.getSelectedIndex());
    }
    
    public void modify() {	
	CpuReserve cr = new CpuReserve();
	cr.serviceClass = _serviceClassChoice.getSelectedIndex();
	cr.period = Long.parseLong(_period.getText())*1000;
	cr.peakProcessingTime = Long.parseLong(_ppt.getText())*1000;
	cr.sustainableProcessingTime = Long.parseLong(_spt.getText())*1000;
	cr.burstTolerance = Long.parseLong(_bt.getText())*1000;
	cr.peakProcessingUtil = new Double(_ppu.getText()).doubleValue();
	CpuJApi.modify(cr, _pid);
    }

    public void setAdaptStrategy() {
	AdaptStrategy as = new AdaptStrategy();
	as.strategy = _adaptStrategyChoice.getSelectedIndex();
	as.windowSize = Integer.parseInt(_windowSize.getText());
	as.alpha = new Double(_alpha.getText()).doubleValue();
	as.max = new Double(_max.getText()).doubleValue();
	as.min = new Double(_min.getText()).doubleValue();
	as.nonconformFreq = new Double(_nonconformFreq.getText()).doubleValue();
	CpuJApi.setAdaptStrategy(as, _pid);
    }
    
    public void actionPerformed(ActionEvent event) {
        Object source = event.getSource();
	if (source == _setBoth) {
	    modify();
	    setAdaptStrategy();	    
	} else if (source == _setAdapt) {
	    setAdaptStrategy();
	} else if (source == _setReserve) {
	    modify();
	} else if (source == _close) {
	    setVisible(false);	
	}
    }
}
