/*
 * Created on Oct 15, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package broker;

import java.util.Timer;
import java.util.TimerTask;

import cpuJApi.CpuReserve;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class Ticket
{
	/**
	 * 	Creation time of the ticket, also used as its index
	 */
	private Long creationTime;
	/**
	 * 	Index of the processor reserved for this ticket
	 */
	private int procIndex;
	/**
	 * 	Cpu Reserve reserved for this ticket
	 */
	private CpuReserve cr;
	/**
	 * PID of the requesting process 
	 */
	private int pid;
	/**
	 * 	Reference to the TicketBuilder controlling generation/expiration of tickets
	 */
	private TicketBuilder tb;
	/**
	 * Leasing time for this ticket
	 */
	private long lease;
	/**
	 * Timer responsible to detect lease's timeout
	 */
	private Timer leaseTimer;
	
	/**
	 * 	Creates a reservation ticket
	 * 
	 * @param procIndex Index of the processor reserved
	 * @param cr Cpu Reserve reserved
	 * @param lease Leasing time for the ticket
	 * @param tb Reference to the builder which must be notified in case this ticket expires
	 */ 
	public Ticket(int pid, int procIndex, CpuReserve cr, long lease, TicketBuilder tb)
	{
		this.creationTime=new Long(System.currentTimeMillis());
		this.pid=pid;
		this.procIndex=procIndex;
		this.cr=cr;
		this.tb=tb;
		
		leaseTimer = new Timer();
		leaseTimer.schedule(new LeaseExpired(), lease);
	}
	
	class LeaseExpired extends TimerTask {
		public void run()
		{
			tb.leaseExpired(creationTime);
			leaseTimer.cancel();
		}
	}
	
	public void cancelTimeout()
	{
			leaseTimer.cancel();
	}
	
	/**
	 * @return Cpu Reserve of the ticket
	 */
	public CpuReserve getCr()
	{
		return cr;
	}

	/**
	 * @return Index (time of creation) of the ticket
	 */
	public Long getCreationTime()
	{
		return creationTime;
	}

	/**
	 * @return Index of the processor reserved for the ticket
	 */
	public int getProcIndex()
	{
		return procIndex;
	}
	
	public Long getId()
	{
		return(this.getCreationTime());
	}
	/**
	 * @return
	 */
	public int getPid()
	{
		return pid;
	}

 }
