package cpuJApi;

public class AdaptStrategy
{
    public int strategy;
    public int windowSize;
    public double alpha;
    public double max;
    public double min;
    public double nonconformFreq;

    public final static int ADAPT_NONE = 0;
    public final static int ADAPT_EXPO = 1;
    public final static int ADAPT_STAT = 2;

    public AdaptStrategy() {
	strategy = ADAPT_NONE;
	windowSize = 0;
	alpha = 0;
	max = 0;
	min = 0;
	nonconformFreq = 0;
    }
    
    public int parse(String args[], int mark) {
	try {
	    if (args[mark].equals("ADAPT_NONE")) {
		strategy = ADAPT_NONE;
		return(1);
	    }
	    else if (args[mark].equals("ADAPT_EXPO")) {
		strategy = ADAPT_EXPO;
		windowSize = Integer.parseInt(args[mark+1]);
		alpha = new Double(args[mark+2]).doubleValue();
	    }
	    else if (args[mark].equals("ADAPT_STAT")) {
		strategy = ADAPT_STAT;
		windowSize = Integer.parseInt(args[mark+1]);
		nonconformFreq = new Double(args[mark+2]).doubleValue();
	    }
	    else
		return(0);
	    
	    if (args.length > mark + 3) {
		if (args[mark+3].equals("RANGE")) {
		    max = new Double(args[mark+4]).doubleValue();
		    min = new Double(args[mark+5]).doubleValue();
		    if (max >  1) {
			max = max*1000;
			min = min*1000;
		    }
		    return(6);
		}
		else if (args[mark+3].equals("MIN")) {
		    min = new Double(args[mark+4]).doubleValue();
		    if (min > 1)
			min = min*1000;
		    return(5);
		}
		else if (args[mark+3].equals("MAX")) {
		    max = new Double(args[mark+4]).doubleValue();
		    if (max > 1)
			max = max*1000;
		    return(5);
		}
	    }
	    return(3);
	} catch (Exception e) {
	    return(0);
	}
    }

    public void usage() {
	System.out.println("[-adapt ADAPT_EXPO windowSize alpha");
	System.out.println("[-adapt ADAPT_EXPO windowSize alpha (MIN min | MAX max | RANGE max min)");
	System.out.println("        ADAPT_STAT windowSize nonconformFreq");
	System.out.println("        ADAPT_STAT windowSize nonconformFreq (MIN min | MAX max | RANGE max min)]");
    }

    public String toString() {
	String r;
	switch (strategy) { 
	case ADAPT_NONE: 
	    return("NONE");
	case ADAPT_EXPO: 
	    r = "EXPO/" + Integer.toString(windowSize) 
		+ "/" + Double3.toString(alpha);
	    break;
	case ADAPT_STAT: 
	    r = "STAT/" + Integer.toString(windowSize) 
		+ "/" + Double3.toString(nonconformFreq);
	    break;
	default: 
	    return("UNKNOWN");
	}
	r = r + "/" + Double3.toString(max/1000)
	    + "/" + Double3.toString(min/1000);
	return(r);
    }

    public String strategyToString() {
	switch (strategy) { 
	case ADAPT_NONE: 
	    return "NONE";
	case ADAPT_EXPO: 
	    return "EXPO";
	case ADAPT_STAT: 
	    return "STAT";
	default: 
	    return "UNKNOWN";
	}
    }
};
