package cpuJApi;

public class CpuJApi {
    
    public static native boolean init();
    public static native boolean reserve(CpuReserve cr, int pid);
    
    public static native boolean modify(CpuReserve cr, int pid);
    public static native boolean free(int pid);
    public static native boolean start(int pid);
    public static native boolean stop(int pid);
    public static native boolean destroy(int pid);
    public static native void yield();
    public static native void svcDestroy();
    public static native boolean setAdaptStrategy(AdaptStrategy as, int pid);
    public static native boolean setApDeadline(long deadline);
    public static native boolean probe(long period);
    public static native boolean probeEnd();
    public static native boolean probeMatch(CpuReserve cr);  
    public static native boolean setRecordProcStats(String fname);
    public static native boolean setRecordSchedStats(String fname, long samplingInterval);

    public static boolean getSchedStats(SchedStats ss) {
	if (_getSchedStats(ss) == false)
	    return(false);
	for (int i=0; i<ss.numProcessor; i++)
	    if (_getSchedStats(ss.processors[i], i) == false)
		return(false);
	return(true);
    };

    public static native boolean isPermNonconform(int pid);
    public static native void printReserveUsage();
    public static native void printAdaptUsage();
    public static native boolean getPidTab(int pidTab[], int len);

    public static boolean reserve(CpuReserve cr) { return(reserve(cr, 0)); };
    public static boolean modify(CpuReserve cr) { return(modify(cr, 0)); };
    public static boolean free() { return(free(0)); };
    public static boolean start() { return(start(0)); };
    public static boolean stop() { return(stop(0)); };
    public static boolean destroy() { return(destroy(0)); };
    public static boolean probe() { return(probe(0)); };
    public static boolean setAdaptStrategy(AdaptStrategy as) { 
	return(setAdaptStrategy(as, 0)); };
    public static boolean setRecordProcStats() { 
	return(setRecordProcStats(null)); };
    public static boolean setRecordSchedStats(String fname) { 
	return(setRecordSchedStats(fname, 1000000)); };
    public static boolean setRecordSchedStats() { 
	return(setRecordSchedStats(null, 1000000)); };
    public static boolean setRecordSchedStats(long samplingInterval) { 
	return(setRecordSchedStats(null, samplingInterval)); };
    public static boolean getProcStats(ProcStats ps) { 
	return(getProcStats(ps, 0)); };
    public static boolean getProc(Proc p) { 
	return(getProc(p, 0)); };
    public static boolean isPermNonconform() { 
	return(isPermNonconform(0)); };   

   
    private static native boolean _getSchedStats(SchedStats ss);
    private static native boolean _getSchedStats(Processor pr, int prid);

    public static boolean getProcStats(ProcStats ps, int pid) {
	return(_getProcStats(ps, ps.cpuReserve, pid));
    }
    
    public static boolean getProc(Proc p, int pid) {
	return(_getProc(p, p.cpuReserve, p.adaptStrategy, pid) &&
	       _getProcStats(p.ps, p.ps.cpuReserve, pid));
    }

    private static native boolean _getProcStats(ProcStats ps, CpuReserve cr, int pid);
    private static native boolean _getProc(Proc p, CpuReserve cr, AdaptStrategy as, int pid);

    static {
	System.out.println("loading libCpuJApi.so");
	System.loadLibrary("CpuJApi");
    }

    final static double M_PI = 3.4159265358979323846;
    public final static int MAX_PROCESSOR = 8;
    public final static int MAX_PROCESS = 8;

    private CpuReserve cr;
    private AdaptStrategy as;
    private int doArg;
    private int doItr;
    private boolean computeFlag;
    private boolean apFlag;
    private boolean sleepFlag;
    private boolean recordPsFlag;
    private long recordPsSamplingInterval;
    private boolean recordSsFlag;
    private long recordSsSamplingInterval;
    private boolean probeFlag;
    private int probeItr;
    private long probePeriod;
    private boolean burstFlag;
    private int burstSize;
    private int numBursts;
    private int burstWindowSize;
    private int late;
    private int apItr;
    private int apNum;
    private long[] apDeadline;
    private int[] apDoMult;


    static void usage() {
	System.out.println("Usage: java CpuJApi");
	System.out.println("  -probe iters period<ms>");
	System.out.println("  -reserve CPU_RT_PCPT period<ms> peakProcessingTime<ms>");
	System.out.println("           CPU_RT_PVPT period<ms> peakProcessingTime<ms> sustainableProcessingTime<ms> burstTolerance<ms>");
	System.out.println("           CPU_RT_ACPU peakProcessingUtil");
	System.out.println("  -adapt   ADAPT_EXPO windowSize alpha");
	System.out.println("           ADAPT_EXPO_UP windowSize alpha");
	System.out.println("           ADAPT_EXPO_RANGE windowSize alpha max min");
	System.out.println("           ADAPT_STAT windowSize nonconformFreq");
	System.out.println("   -do compute arg iters burstSize numBursts windowSize");
	System.out.println("       sleep <ms> iters burstSize numBursts windowSize");
	System.out.println("   -ap n apDeadline-1<ms> doMul-1 apDeadline-2<ms> doMul-2... apDeadline-n<ms> doMul-n");
	System.out.println("   -record none");
	System.out.println("           ps ");
	System.out.println("           ss samplingInterval<ms>");
	System.exit(0);
    }

    static void error() {
	error(null);
    }
    
    
    static void error(String s) {
	System.out.println("Error: " + s);
	destroy();
	System.exit(0);
    }
    

    CpuJApi() {
	cr = new CpuReserve();
	as = new AdaptStrategy();
	doItr = 0;
	computeFlag = false;
	apFlag = false;
	sleepFlag = false;
	recordPsFlag = true;
	recordSsFlag = true;
	recordSsSamplingInterval = 1000000;
	probeFlag = false;
	probeItr = 0;
	probePeriod = 0;
	burstFlag = false;
	burstSize = 0;
	numBursts = 0;
	burstWindowSize = 0;
	late = 0;
	apItr = 0;
	apNum = 0;
	apDeadline = null;
	apDoMult = null;
    }

    double fact(int f) {
	double res;
	res = 1;
	for (int i=2;i<=f;i++)
	    res = res*i;
	return(res);
    }

    double power(double x, int p) {
	double res;
	res = 1;
	for (int i=1;i<=p;i++)
	    res = res*x;
	return(res);
    }

    void compute(int num) {
	int i, prec;
	double x, sign;
	double sin_val, cos_val;
	
	for (i=0; i<num; i++) {
	    for (x = 0.0; x <= 2*M_PI; x = x + 0.001) {
		sin_val = 0;
		cos_val = 0;
		sign = 1;
		for (prec=1; prec<=10; prec++) {
		    sin_val = sin_val + sign*power(x,prec)/fact(prec);
		    cos_val = cos_val + sign*power(x,prec-1)/fact(prec-1);
		    sign = (-1)*sign;
		}
	    }
	} 
    }

    void doOneItr(int i) {
	int mult = 1;
	if (cr.serviceClass == CpuReserve.CPU_RT_ACPU) {
	    setApDeadline(apDeadline[(i+1)%apNum]);
	    mult = apDoMult[i%apNum];
	}
	
	if ((burstFlag) && (i%burstWindowSize < numBursts)) {
	    if (computeFlag)
		for (int k=0; k<mult; k++) 
		    compute(burstSize);
	    else {
		try {
		    Thread.sleep(burstSize * mult/1000);
		} catch(InterruptedException e) {
		}
	    }
	}
	else {
	    if (computeFlag)
		for (int k=0; k<mult; k++) 
		    compute(doArg);
	    else {
		try {
		    Thread.sleep(doArg * mult/1000);
		} catch(InterruptedException e) {
		}
	    }
	}
    }

    public void sample(String[] args) {
	int mark = 0;
	int n;
	String fname = null;

	while (mark < args.length) {
	    if (args[mark].equals("-probe")) {
		probeFlag = true;
		probeItr = Integer.parseInt(args[mark+1]);
		probePeriod = Long.parseLong(args[mark+2]) * 1000;
		mark += 3;
	    }
	    else if (args[mark].equals("-reserve")) {
		mark++;
		if ((n = cr.parse(args, mark)) == 0) {
		    usage();
		}
		mark += n;
	    } else if (args[mark].equals("-do")) {
		if (args[mark+1].equals("sleep"))
		    sleepFlag = true;
		else if (args[mark+1].equals("compute"))
		    computeFlag = true;
		else
		    usage();
		doArg = Integer.parseInt(args[mark+2]);
		doItr = Integer.parseInt(args[mark+3]);
		mark += 4;
		if ((args.length > mark) && (args[mark].startsWith("-") == false)) {
		    burstFlag = true;
		    burstSize = Integer.parseInt(args[mark]);
		    numBursts = Integer.parseInt(args[mark+1]);
		    burstWindowSize = Integer.parseInt(args[mark+2]);
		    mark += 3;
		}
		if (sleepFlag) {
		    doArg *= 1000;
		    burstSize *= 1000;
		}
	    } else if (args[mark].equals("-adapt")) {
		mark++;
		if ((n = as.parse(args, mark)) == 0) {
		    usage();
		}
		mark += n;
	    } else if (args[mark].equals("-fname")) {
		fname = new String(args[mark+1]);
		mark += 2;
	    }
	    else if (args[mark].equals("-record")) {
		if (args[mark+1].equals("none")) {
		    recordPsFlag = false;
		    recordSsFlag = false;
		    mark += 2;
		}
		else if (args[mark+1].equals("ps")) {
		    recordSsFlag = false;
		    mark += 2;
		}
		else if (args[mark+1].equals("ss")) {
		    recordPsFlag = false;
		    recordSsSamplingInterval = Long.parseLong(args[mark+2]) * 1000;
		    mark += 3;
		}
	    }
	    else if (args[mark].equals("-ap")) {
		apFlag = true;
		apNum = Integer.parseInt(args[mark+1]);
		apDeadline = new long[apNum];
		apDoMult = new int[apNum];
		mark += 2;
		for (int j=0; j<apNum; j++) {
		    apDeadline[j] = Long.parseLong(args[mark]) * 1000;
		    apDoMult[j] = Integer.parseInt(args[mark+1]);
		    mark += 2;
		}
	    }
	    else 
		usage();
	}
	
	if ((doItr < 1) ||
	    (probeFlag && ((probePeriod==0 && !apFlag) || (probePeriod>0 && apFlag))) ||
	    ((cr.serviceClass == CpuReserve.CPU_RT_ACPU) && (apNum == 0)) ||
	    (!computeFlag && !sleepFlag))
	    usage();
	
	
	/**************  Initialization Phase **************/
	if (!init())
	    error("init()");
	System.out.println("init()");

	if (recordPsFlag) {
	    if (!setRecordProcStats(fname))
		error("setRecordProcStats()");
	    System.out.println("setRecordProcStats()");
	}
	if (recordSsFlag) {
	    if (!setRecordSchedStats(fname, recordSsSamplingInterval))
		error("setRecordSchedStats()");
	    System.out.println("setRecordSchedStats()");
	}
	
	/**************  Probing Phase **************/
	if (probeFlag) {
	    if (!probe(probePeriod))
		error("probe()");
	    System.out.println("probe()");

	    if (!start())
		error("start()");	    
	    System.out.println("start()");

	    for (int i=0; i<probeItr; i++) {
		doOneItr(i);
		yield();
	    }
	    
	    if (!stop())
		error("stop()");
	    System.out.println("stop()");

	    if (!probeEnd())
		error("probeEnd()");
	    System.out.println("probeEnd()");

	    if (!probeMatch(cr))
		error("probeMatch()");	    
	    System.out.println("probeMatch()");

	    System.out.println("Probe Result:");
	    System.out.println(cr.toString());
	}
	
	/**************  Reservation Phase **************/
	if (!reserve(cr))
	    error("svc rejects reservation");
	System.out.println("reserve()");
	
	if (!setAdaptStrategy(as))
	    error("svc rejects adaptation strategy");
	System.out.println("setAdaptStrategy()");

	if (cr.serviceClass == CpuReserve.CPU_RT_ACPU) {
	    if (!setApDeadline(apDeadline[0]))
		error("setApDeadline()");
	    System.out.println("setApDeadline()");
	}
	
	/**************  Execution Phase **************/
	if (!start())
	    error("svc refuses to start it");
	System.out.println("start()");
	
	for (int i=0; i<doItr; i++) {
	    doOneItr(i);
	    yield();
	    System.out.println("iteration " + i);
	}
	
	if (!stop())
	    error("stop()");
	System.out.println("stop()");

	if (!free())
	    error("free()");
	System.out.println("free()");

	destroy();
    }


    public static void main(String[] args) {
	CpuJApi cpu = new CpuJApi();
	cpu.sample(args);
    }
}
