package cpuJApi;

public class CpuReserve {
    public int serviceClass;
    public long period;  // CPU_RT_PCPT, CPU_RT_PVPT
    public long peakProcessingTime;  // CPU_RT_PCPT, CPU_RT_PVPT
    public long sustainableProcessingTime;  // CPU_RT_PVPT
    public long burstTolerance;      // CPU_RT_PVPT
    public double peakProcessingUtil;  // CPU_RT_ACPU
    public static final int CPU_RT_PCPT = 0;
    public static final int CPU_RT_PVPT = 1;
    public static final int CPU_RT_ACPU = 2;
    public static final int CPU_RT_EVENT = 3;
    public static final int CPU_RT_PCCPT = 4;

    public CpuReserve() {
	serviceClass = CPU_RT_PCPT;
	period = 0;
	peakProcessingTime = 0;
	sustainableProcessingTime = 0;
	burstTolerance = 0;
	peakProcessingUtil = 0;
    }

    public String toString() {
	String r;
	switch (serviceClass) {
	case CPU_RT_PCPT:
	    r = "RT_PCPT";
	    sustainableProcessingTime = 0;
	    burstTolerance = 0;
	    break;
	case CPU_RT_PVPT:
	    r = "RT_PVPT";
	    peakProcessingUtil = 0;
	    break;
	case CPU_RT_ACPU:
	    r = "RT_ACPU";
	    sustainableProcessingTime = 0;
	    burstTolerance = 0;
	    break;
	case CPU_RT_EVENT:
	    r = "RT_EVENT";
	    sustainableProcessingTime = 0;
	    burstTolerance = 0;
	    peakProcessingUtil = 0;
	    break;
	case CPU_RT_PCCPT:
	    r = "RT_PCCPT";
	    sustainableProcessingTime = 0;
	    burstTolerance = 0;
	    peakProcessingUtil = 0;
	    break;
	default:
	    r = "UNKNOWN";
	    break;
	}
	return(r + "/" + period/1000 + "/" + peakProcessingTime/1000 + 
	       "/" + sustainableProcessingTime/1000 + "/" + burstTolerance/1000 + 
	       "/" + Double3.toString(peakProcessingUtil));
    }

    public String serviceClassToString() {
	switch (serviceClass) {
	case CPU_RT_PCPT:
	    return "RT_PCPT";
	case CPU_RT_PVPT:
	    return "RT_PVPT";
	case CPU_RT_ACPU:
	    return "RT_ACPU";
	case CPU_RT_EVENT:
	    return "RT_EVENT";
	default:
	    return "UNKNOWN";
	}
    }

    
    public int parse(String[] args, int mark) {
	try {
	    if (args[mark].equals("CPU_RT_PCPT")) {
		serviceClass = CPU_RT_PCPT;
		period = Long.parseLong(args[mark+1]) * 1000; 
		peakProcessingTime = Long.parseLong(args[mark+2]) * 1000; 
		return(3);
	    }
	    else if (args[mark].equals("CPU_RT_PVPT")) {
		serviceClass = CPU_RT_PVPT;
		period = Long.parseLong(args[mark+1]) * 1000; 
		peakProcessingTime = Long.parseLong(args[mark+2]) * 1000; 
		sustainableProcessingTime = Long.parseLong(args[mark+3]) * 1000; 
		burstTolerance = Long.parseLong(args[mark+4]) * 1000; 
		return(5);
	    }
	    else if (args[mark].equals("CPU_RT_ACPU")) {
		serviceClass = CPU_RT_ACPU;
		peakProcessingUtil = new Double(args[mark+1]).doubleValue(); 
		return(2);
	    }
	    else if (args[mark].equals("CPU_RT_EVENT")) {
		serviceClass = CPU_RT_EVENT;
		peakProcessingTime = Long.parseLong(args[mark+1]);
		return(2);
	    }
	    else
		return(0);
	} catch (Exception e) {
	    return(0);
	}
    }

    public void usage() {
	System.out.println("[-reserve CPU_RT_PCPT period<ms> peakProcessingTime<ms>");
	System.out.println("          CPU_RT_PVPT period<ms> peakProcessingTime<ms>");
	System.out.println("                      sustainableProcessingTime<ms> burstTolerance<ms>");
	System.out.println("          CPU_RT_ACPU peakProcessingUtil]");
	System.out.println("          CPU_RT_Event peakProcessingTime<ms>]");
    }
};
