/*
 * Created on Dec 1, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package negotiator;

import java.io.IOException;
import java.io.Serializable;
import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import reflector.RoamingStrategy;
import reflector.RoamingStrategyLinear;
import reflector.RoamingStrategyParallel;
import CORBAbroker.QMLcompiler.Parser;

import com.ibm.aglet.Aglet;
import com.ibm.aglet.AgletException;
import com.ibm.aglet.AgletProxy;
import com.ibm.aglet.InvalidAgletException;
import com.ibm.aglet.Message;
import com.ibm.aglet.MessageException;
import com.ibm.aglet.NotHandledException;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class Negotiator extends Aglet implements Serializable
{
	transient URL specFile;
	
	Vector specifications;
	Vector reservations;
	Integer requesterPid;
	AgletProxy application;
	
	int roamingStrategy;
	
	int numberOfReplies;
	int repliesReceived; 
	
	Vector results;
	boolean negotiationIsOver;
		
	public Negotiator()
	{}
	
	/* (non-Javadoc)
	 * @see com.ibm.aglet.Aglet#onCreation(java.lang.Object)
	 */
	public void onCreation(Object init)
	{
		NegotiatorInit negInit=(NegotiatorInit)init;
		results=new Vector();
		this.specFile=negInit.getFileName();
		this.requesterPid=new Integer(negInit.getPid());
		
		this.application = negInit.getApplication();
		
		this.negotiationIsOver=false;
		
		
		roamingStrategy=negInit.getRoamingStrategy();
		
		try
		{
			specifications=Parser.parse(specFile);
			// Specifications follows the format "program_name -reserve cpu_class value[s]"
			// For negotiation, I need only "cpu_class value[s]" on a String[] array
			reservations = convertToReservations(specifications);
		} catch (IOException e)
		{
			System.out.println("File "+specFile+" not found.");
			e.printStackTrace();
		}	
	}
		
	/**
	 * @param specifications
	 * @return
	 */
	private Vector convertToReservations(Vector specifications)
	{
		Vector res=new Vector();
		
		// Cuts out the beggining of the specification string
		for (Enumeration e = specifications.elements() ; e.hasMoreElements() ;) 
		{
			String spec=(String)e.nextElement();
			spec=spec.trim();
				
			String[] reserv=new String[5];
			int begin=0;
			int end=spec.indexOf(' ');
			if(end==-1)
				end=spec.length();
			int param=0;
			while((end-begin)>0)
			{
				param++;
				if(param>2)
				{
					reserv[param-3]=new String(spec.substring(begin, end).trim());
				}
				begin=end;
				end=spec.indexOf(' ', end+1);
				if(end==-1)
					end=spec.length();
			}
			
			param++;
			while((param-3)<5)
			{
				reserv[param-3]=new String();
				param++;
			}
			res.add(reserv);
		}
		
		return(res);
	}

	public void adapt()
	{
//		System.out.println("[NEGOTIATOR] "+specifications);
//		System.out.println("[NEGOTIATOR] "+requesterPid);
		
		RoamingStrategy rs = null;
				
		switch(roamingStrategy)
		{
			case RoamingStrategy.LINEAR_ROAMING:
				System.out.println("[NEGOTIATOR] ******* LINEAR ROAMING *******");
				rs=new RoamingStrategyLinear(reservations, requesterPid, getProxy());
				break;
			case RoamingStrategy.PARALLEL_ROAMING:
				System.out.println("[NEGOTIATOR] ******* PARALLEL ROAMING *******");
				rs=new RoamingStrategyParallel(reservations, requesterPid, getProxy());
				break;
		}
			
		Vector inits=rs.definePath();
		numberOfReplies = rs.getNumberOfReplies();
		
		System.out.println("[NEGOTIATOR] Negotiator is going to create "+inits.size()+" agents for negotiation");
		
		// Number of replies that must be received. One for each agent that was created
		
		repliesReceived=0;
				
		for(Iterator it=inits.iterator();it.hasNext();)
		{
			try
			{				
				getAgletContext().createAglet(getCodeBase(), "negotiator.NegotiationAgent", (NegotiationAgentInit)it.next());
				System.out.println("[NEGOTIATOR] Created agent");
			} catch (IOException e1)
			{
				e1.printStackTrace();
			} catch (AgletException e1)
			{
				e1.printStackTrace();
			} catch (ClassNotFoundException e1)
			{
				e1.printStackTrace();
			} catch (InstantiationException e1)
			{
				e1.printStackTrace();
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.aglet.Aglet#handleMessage(com.ibm.aglet.Message)
	 */
	public boolean handleMessage(Message msg)
	{
		if(msg.sameKind("startNegotiation"))
		{
			adapt();
			return(true);
		}
		if(msg.sameKind("result"))
		{
			System.out.println("[NEGOTIATOR] Receiving result...");
			NegotiationResult res=(NegotiationResult)msg.getArg();
			repliesReceived++;
			System.out.println("[NEGOTIATOR] Number of replies received = " + repliesReceived);
			System.out.println("[NEGOTIATOR] Waiting for " + (numberOfReplies-repliesReceived) + " replies");
			
			if(res!=null)
			{
				System.out.println("[NEGOTIATOR] Negotiation finished on host "+res.getHost());
					
				if(res.getTicket()!=-1)
				{
					this.results.add(res);
				}
			}
			
			if(repliesReceived==numberOfReplies)
			{
				System.out.println("[NEGOTIATOR] Sending negotiation results!!!!");
				try
				{
					application.sendMessage(new Message("negotiationResults", results));
				} catch (InvalidAgletException e)
				{
					e.printStackTrace();
				} catch (NotHandledException e)
				{
					e.printStackTrace();
				} catch (MessageException e)
				{
					e.printStackTrace();
				}				
			}
					
			return(true);
		}
		if(msg.sameKind("getResult"))
		{
			if(repliesReceived!=numberOfReplies)
				msg.sendReply(null);
			else
				msg.sendReply(results);
			return(true);
		}
		
		return(false);
	}

}
