/*
 * Created on Mar 5, 2004
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package reflector;

import java.io.Serializable;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.Vector;

import negotiator.NegotiationResult;

import com.ibm.aglet.AgletProxy;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
//public abstract class AdaptStrategy extends Thread implements Serializable
public class AdaptStrategy extends Thread implements Serializable
{
	protected AgletProxy proxy;
	protected AgletProxy negotiator;
	protected String localhost;
	
	protected Vector destinations;
	
	public AdaptStrategy(AgletProxy reflectorProxy, AgletProxy neg)
	{
		proxy=reflectorProxy;
		negotiator=neg;
		try
		{
			localhost = InetAddress.getLocalHost().getHostName();
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
	}
	
	public AdaptStrategy()
	{
	}
	
	/*public void negotiate()
	{
		Vector reply=null;
		
		try
		{
			System.out.println("Runs negotiator to find better location");
			
			// Time metrics
			long begin;
			begin = System.currentTimeMillis();
			
			negotiator.sendMessage(new Message("startNegotiation"));
												
			do
			{
				System.out.println("[ADAPTATION] Waiting 7 seconds for negotiation results");
				sleep(7000);
				System.out.println("[ADAPTATION] Getting negotiation results");
				reply = (Vector)negotiator.sendMessage(new Message("getResult"));
			}while(reply.size()<1);
			
			while(reply==null)
			{
				reply = (Vector)negotiator.sendMessage(new Message("getResult"));
				sleep(10);
			}
			
			long end;
			end = System.currentTimeMillis();
			
			System.out.println("[ADAPTATION] Time spent negotiating = ******* "+(end-begin)+" miliseconds *******");
			System.out.println("[ADAPTATION] Results:");
			for(Iterator i=reply.iterator();i.hasNext();)
			{
				NegotiationResult r=(NegotiationResult)i.next();
				System.out.println(r.toString());
			}
		} 
		catch (InvalidAgletException e)
		{
			System.out.println("[ERROR] Invalid negotiator aglet.");
			e.printStackTrace();
		} catch (NotHandledException e)
		{
			System.out.println("[ERROR] Negotiator failed to handle message.");
			e.printStackTrace();
		} catch (MessageException e)
		{
			System.out.println("[ERROR] Error creating message.");
			e.printStackTrace();
		} catch (InterruptedException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return(reply);		
	}*/
	
	public void run()
	{
	}

	/**
	 * @param destinations
	 * @return
	 */
	protected NegotiationResult bestOffer(Vector destinations)
	{	
		if(destinations.size()!=0)
		{
			Iterator it=destinations.iterator();
			NegotiationResult winner=(NegotiationResult)it.next();
		
			while(it.hasNext())
				winner = maxReservation(winner, (NegotiationResult)it.next());
			
			return(winner);
		}
			
		// No destinations found, negotiation failed
		else
			return(null);
	}

	/**
	 * @param negres1
	 * @param negres2
	 * @return
	 */
	protected NegotiationResult maxReservation(NegotiationResult negres1, NegotiationResult negres2)
	{
		if(negres1==null && negres2==null)
			return null;
		
		if(negres1==null)
			return(negres2);
		
		if(negres2==null)
			return(negres1);
	
		// Actual comparison
		String resType1 = negres1.getReservation()[0];
		String resType2 = negres2.getReservation()[0];
	
		// Can't compare oranges to bananas (different reservation types)
		if(!resType1.equals(resType2))
			return(null);
		
		// For each existing type of reservation, a different criteria must be used
		if(resType1.equals("CPU_RT_PCPT") || resType1.equals("CPU_RT_PCCPT") || resType1.equals("CPU_RT_EVENT"))
		{
			int period1 = Integer.parseInt(negres1.getReservation()[1]);
			int peak1 =  Integer.parseInt(negres1.getReservation()[2]);
			int period2 = Integer.parseInt(negres2.getReservation()[1]);
			int peak2 =  Integer.parseInt(negres2.getReservation()[2]);
		
			if(((double)peak1/(double)period1) > ((double)peak2/(double)period2))
				return(negres1);
			if(((double)peak1/(double)period1) < ((double)peak2/(double)period2))
				return(negres2);
			if(((double)peak1/(double)period1) == ((double)peak2/(double)period2))
			{
				if(period1 <= period2)
					return(negres1);
				else
					return(negres2);
			}
		}
	
		if(resType1.equals("CPU_RT_ACPU"))
		{
			int peak1 =  Integer.parseInt(negres1.getReservation()[1]);
			int peak2 =  Integer.parseInt(negres2.getReservation()[1]);

			if(peak1 >= peak2)
				return(negres1);
			if(peak1 < peak2)
				return(negres2);
		}
	
		if(resType1.equals("CPU_RT_PVPT"))
		{
			int period1 = Integer.parseInt(negres1.getReservation()[1]);
			int peak1 =  Integer.parseInt(negres1.getReservation()[2]);
			int sustainable1 = Integer.parseInt(negres1.getReservation()[3]);
			int burst1 = Integer.parseInt(negres1.getReservation()[4]);
		
			int period2 = Integer.parseInt(negres2.getReservation()[1]);
			int peak2 =  Integer.parseInt(negres2.getReservation()[2]);
			int sustainable2 = Integer.parseInt(negres2.getReservation()[3]);
			int burst2 = Integer.parseInt(negres2.getReservation()[4]);

			if(((double)sustainable1/(double)period1) > ((double)sustainable2/(double)period2))
				return(negres1);
			if(((double)sustainable1/(double)period1) < ((double)sustainable2/(double)period2))
				return(negres2);
			if(((double)sustainable1/(double)period1) == ((double)sustainable2/(double)period2))
			{
				if(peak1 > peak2)
					return(negres1);
				if(peak1 < peak2)
					return(negres2);
				if(peak1 == peak2)
				{
					if(burst1 >= burst2)
						return(negres1);
					else
						return(negres2);
				}
			}
		}
	
		System.out.println("[AdaptMigrate] Unknown reservation type");
		return null;
	}	
	
	/**
	 * @param vector
	 */
	public void setDestinations(Vector vector)
	{
		destinations = vector;
		System.out.println("[ADAPTATION] Results:");
		for(Iterator i=destinations.iterator();i.hasNext();)
		{
			NegotiationResult r=(NegotiationResult)i.next();
			System.out.println(r.toString());
		}
	}

}
