/*
 * Created on Jul 28, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package reflector;

import java.io.PrintWriter;
import java.io.Serializable;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class CountingPrintWriter implements Serializable
{
	private long bytes;
	private long totalBytes;
	
	private int resolution;
	private final int DEFAULT_RESOLUTION = 5; // Summarizes data every 5 seconds
	
	private long startTime;
	private long intervalStartTime;
	
	private double currentBandwidth;
	private double averageBandwidth;
	
	private PrintWriter print;

	/**
	 * @param out
	 */
	public CountingPrintWriter(PrintWriter out)
	{
		this.print = out;
		initVariables();
	}
	
	public CountingPrintWriter()
	{}

	void initVariables()
	{
		this.bytes = this.totalBytes = 0;
		this.resolution = this.DEFAULT_RESOLUTION;
		this.intervalStartTime = this.startTime = System.currentTimeMillis();
		this.currentBandwidth = this.averageBandwidth = 0.0;
	}
	
	
	/* (non-Javadoc)
	 * @see java.io.PrintWriter#print(char[])
	 */
	public void print(char[] s)
	{
		dataFlow(s.length);
		
		print.print(s);
	}

	public void dataFlow(int numOfBytes)
	{
		bytes += numOfBytes;
		totalBytes += numOfBytes;
		
		long currentTime = System.currentTimeMillis();
		
		// Avoids division by zero
		if(currentTime - startTime < 1000)
			return;
			
		// Checks if an interval has passed, and updates the values
		if(currentTime - intervalStartTime > resolution * 1000)
		{
			this.currentBandwidth = (bytes*8)/resolution;
			this.averageBandwidth = (totalBytes*8)/((currentTime - startTime)/1000);
			
			intervalStartTime = currentTime;
			bytes = 0;			
		}
	}
	
	
	/**
	 * @return
	 */
	public double getAverageBandwidth()
	{
		return averageBandwidth;
	}

	/**
	 * @return
	 */
	public double getCurrentBandwidth()
	{
		return currentBandwidth;
	}

}
