/*
 * Created on Jun 30, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package reflector;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

import org.tritonus.share.sampled.AudioFileTypes;
import org.tritonus.share.sampled.Encodings;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class MixerConnectionHandler extends Thread implements Serializable
{
	transient private boolean keepSending;
	transient Socket connection;
	transient URL streamingServerAddr1;
	transient URL streamingServerAddr2;
	transient Socket streamingServer1;
	transient Socket streamingServer2;
	transient InputStream input1;
	transient InputStream input2;
	
	transient PrintWriter outputToServer1;
	transient PrintWriter outputToServer2;
	
	CountingPrintWriter codeWriter;
	transient BufferedOutputStream outputToClient;
	
	BandwidthDialog codeDialog;
	transient BandwidthDialog dlg;
	
	String server1Address;
	String server2Address;
	
	private static final int BUFFER_SIZE = 4608; //128 * 1024;
	private static final int HTTP_HEADER_SIZE = 205;
	private static final AudioFormat.Encoding	MPEG1L3 = Encodings.getEncoding("MPEG1L3");
	
	public MixerConnectionHandler(String server1Address, String server2Address, Socket connection)
	{
		this.connection=connection;
		this.keepSending=true;
		
		this.server1Address=server1Address;
		this.server2Address=server2Address;
		this.codeDialog = new BandwidthDialog(null);
		this.codeWriter = new CountingPrintWriter(null);
	}
	
	public void run()
	{
		try
		{
			streamingServerAddr1 = new URL(server1Address);
			streamingServerAddr2 = new URL(server2Address);

			String host1=streamingServerAddr1.getHost();
			String host2=streamingServerAddr2.getHost();
			int port1=streamingServerAddr1.getPort();
			if(port1==-1)
				port1=80;
			int port2=streamingServerAddr2.getPort();
			if(port2==-1)
				port2=80;

			System.out.println("[DEBUG] Addr -> "+host1);
			System.out.println("[DEBUG] Port -> "+port1);
			System.out.println("[DEBUG] File -> "+streamingServerAddr1.getFile());

			System.out.println();
			System.out.println("[DEBUG] Addr -> "+host2);
			System.out.println("[DEBUG] Port -> "+port2);
			System.out.println("[DEBUG] File -> "+streamingServerAddr2.getFile());
							
			streamingServer1=new Socket(host1, port1);
			streamingServer2=new Socket(host1, port1);
			
			outputToClient = new BufferedOutputStream(connection.getOutputStream());
			
			input1 = streamingServer1.getInputStream();
			input2 = streamingServer2.getInputStream();
			
			outputToServer1 = new PrintWriter(streamingServer1.getOutputStream(), true);
			outputToServer2 = new PrintWriter(streamingServer2.getOutputStream(), true);

			// Begins http connection
			System.out.println("GET "+streamingServerAddr1.getFile()+" HTTP/1.0\r \n");
			System.out.println("GET "+streamingServerAddr2.getFile()+" HTTP/1.0\r \n");

			outputToServer1.println("GET "+streamingServerAddr1.getFile()+" HTTP/1.0\r \n");
			outputToServer2.println("GET "+streamingServerAddr2.getFile()+" HTTP/1.0\r \n");
		} catch (MalformedURLException e)
		{
			e.printStackTrace();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
			
		// Shows the dialog with bandwidth statistics
		//dlg = new BandwidthDialog(outputToClient);
		//dlg.start();
	
		try
		{
			// Java sound stuff: get Audio streams, discover sound format, etc.
			AudioInputStream ais1 = null;
			AudioInputStream ais2 = null;
			AudioInputStream aisEnc = null;
			
			File test = new File("teste.mp3");
			FileOutputStream fos = new FileOutputStream(test);
			
			try
			{
				ais1=AudioSystem.getAudioInputStream(streamingServerAddr1);
				ais2=AudioSystem.getAudioInputStream(streamingServerAddr2);
			} 
			catch (UnsupportedAudioFileException e2)
			{
				e2.printStackTrace();
			}
	
			AudioFormat af1=ais1.getFormat();
			AudioFormat af2=ais2.getFormat();
				
			AudioFormat sourceFormat1 = ais1.getFormat();
			AudioFormat targetFormat1 = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, sourceFormat1.getSampleRate(),
							16, sourceFormat1.getChannels(), sourceFormat1.getChannels() * (16/8), sourceFormat1.getSampleRate(),
							false);
			System.out.println("AudioPlayer.main(): source format: " + sourceFormat1);
			System.out.println("AudioPlayer.main(): target format: " + targetFormat1);
			ais1 = AudioSystem.getAudioInputStream(targetFormat1, ais1);
			af1 = ais1.getFormat();
			System.out.println("AudioPlayer.main(): converted AIS: " + ais1);
			System.out.println("AudioPlayer.main(): converted format: " + af1);
	
			AudioFormat sourceFormat2 = ais2.getFormat();
			AudioFormat targetFormat2 = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, sourceFormat2.getSampleRate(),
						16, sourceFormat2.getChannels(), sourceFormat2.getChannels() * (16/8), sourceFormat2.getSampleRate(),
						false);
			System.out.println("AudioPlayer.main(): source format: " + sourceFormat2);
			System.out.println("AudioPlayer.main(): target format: " + targetFormat2);
			ais2 = AudioSystem.getAudioInputStream(targetFormat2, ais2);
			af2 = ais2.getFormat();
			System.out.println("AudioPlayer.main(): converted AIS: " + ais2);
			System.out.println("AudioPlayer.main(): converted format: " + af2);
			
			System.setProperty("tritonus.lame.quality", "lowest");
			System.setProperty("tritonus.lame.vbr", "false");
			System.setProperty("tritonus.lame.bitrate", "8");
			
			// Reads http header information
			byte[] header1 = new byte[HTTP_HEADER_SIZE];
			byte[] header2 = new byte[HTTP_HEADER_SIZE];
			
			input1.read(header1);
			input2.read(header2);
			
			// Passes header to client
			outputToClient.write(header1);
			
			int nBytesRead=0;
			int nBytesRead1=0;
			int nBytesRead2=0;
			
			AudioInputStream aisMixedRaw = null;
			ByteArrayInputStream mixedRaw = null;
			//AudioInputStream aisEnc = null;
		
			byte[] readByte1 = new byte[BUFFER_SIZE];
			byte[] readByte2 = new byte[BUFFER_SIZE];

			byte[] mixedByte = new byte[BUFFER_SIZE];
			byte[] mixedEncodedByte = new byte[BUFFER_SIZE];
			
			while (nBytesRead != -1)
			{
				long ini=System.currentTimeMillis();
				
				try
				{
					nBytesRead1 = ais1.read(readByte1);
					nBytesRead2 = ais2.read(readByte2);
					nBytesRead = (nBytesRead1 > nBytesRead2) ? nBytesRead1 : nBytesRead2;  
														
					for(int x=0; x<mixedByte.length; x++)
					{
						mixedByte[x]=(byte) ((readByte1[x]+readByte2[x])/2);
					}
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
	
				if (nBytesRead > 0)
				{
					mixedRaw = new ByteArrayInputStream(mixedByte);

					aisMixedRaw = new AudioInputStream(mixedRaw, af1, AudioSystem.NOT_SPECIFIED);
					aisEnc = encode(aisMixedRaw, MPEG1L3);
					
					//long ini4 = System.currentTimeMillis();
					int nBytesEncoded = aisEnc.read(mixedEncodedByte);
					//AudioSystem.write(aisEnc, AudioFileTypes.getType("MP3", "mp3"), outputToClient);
					//System.out.println("Reading took "+(System.currentTimeMillis()-ini4));
					//ini4 = System.currentTimeMillis();
					System.out.println("BytesEncoded "+nBytesEncoded);
					outputToClient.write(mixedEncodedByte, 0, nBytesEncoded);
					//System.out.println("Output took "+(System.currentTimeMillis()-ini4));
				}
				
				//System.out.println("Loop took "+(System.currentTimeMillis()-ini));
			}
		} catch (IOException e1)
		{
			e1.printStackTrace();
		}
	}
	
	public AudioInputStream encode(AudioInputStream ais, AudioFormat.Encoding targetEncoding)
	{
		AudioFormat sourceFormat=ais.getFormat();
		//System.out.println("Input format: " + sourceFormat);
	
		// construct a converted stream
		AudioInputStream targetStream = null;
		if (!AudioSystem.isConversionSupported(targetEncoding, sourceFormat)) 
		{
			System.out.println("direct conversion not supported");
			AudioFormat intermediateFormat = new AudioFormat(
												  AudioFormat.Encoding.PCM_SIGNED,
												  sourceFormat.getSampleRate(),
												  16,
												  sourceFormat.getChannels(),
												  2 * sourceFormat.getChannels(), // frameSize
												  sourceFormat.getSampleRate(),
												  false);
			if (AudioSystem.isConversionSupported(intermediateFormat, sourceFormat)) 
			{
				// intermediate conversion is supported
				System.out.println("intermediate conversion supported");
				ais = AudioSystem.getAudioInputStream(intermediateFormat, ais);
			}
		}
		
		targetStream = AudioSystem.getAudioInputStream(targetEncoding, ais);
		if (targetStream == null)
		{
			System.out.println("conversion not supported");
		}

		/*System.out.println("Got converted AudioInputStream: " + targetStream.getClass().getName());
		System.out.println("Output format: " + targetStream.getFormat());*/
 
		 /*byte[] bTst = new byte[8];
		 try
		{
			targetStream.read(bTst, 0, bTst.length);
		} catch (IOException e)
		{
			e.printStackTrace();
		}*/
 
		return targetStream;
	}
	
	public void stopSending()
	{
		this.keepSending=false;
		dlg.stopDlg();
	}
	
}
