// A PCPT process is conforming if its usage time does not exceed peakProcessing_per_period every period.
// If the reservation is accepted, the CPU server guarantees peakProcessing_per_period of usage every period
type CPU_RT_PCPT = contract {
    period: decreasing numeric usec;
    peakProcessing_per_period: decreasing numeric;    
};

// A PCCPT process does not need to specify yield().  The CPU server will always allocate peakProcessing_per_period 
// every period.  It is useful for running any unmodified executable.
type CPU_RT_PCCPT = contract {
    period: decreasing numeric usec;
    peakProcessing_per_period: decreasing numeric;    
};

// A PVPT process is conforming if its usage time does not exceed peakProcessing_per_period every period and its
// accumulative burst (defined as the usage exceeding its sustainableProcessing_per_period) does not exceed burstTolerance.
// If the reservation is accepted, the CPU server guarantees sustainableProcessing_per_period of usage every period.
// The CPU server will service the conforming bursts at the highest priority in its overrun partition.
type CPU_RT_PVPT = contract {
    period: decreasing numeric usec;
    peakProcessing_per_period: decreasing numeric;
    sustainableProcessing_per_period: decreasing numeric;
    burstTolerance: decreasing numeric;
};

// An ACPU process is conforming if its usage percentage does not exceed peakProcessing_in_period.  An ACPU process
// must specify the length of the relative deadline in its next iteration using the API call setApDeadline().  If the
// reservation is accepted, the CPU server guarantees peakProcessing_in_period * (relative deadline) every iteration.
type CPU_RT_ACPU = contract {
    peakProcessingUtil: decreasing numeric;
};

// An Event reservation is valid for only one iteration of length period. An Event process is conforming if its usage
// does not exceed peakProcessing_in_period within its period.
type CPU_RT_EVENT = contract {
    period: decreasing numeric usec;
    peakProcessing_in_period: decreasing numeric;    
};

processamentoIdeal = CPU_RT_PCPT contract {
    period = 40;
//    period = 100;
//    peakProcessing_per_period = 25;
    peakProcessing_per_period = 40;
};

processamentoComDimensaoErrada = CPU_RT_PCPT contract {
    periodMax = 40;
    peakProcessing_per_period = 25;
};

processamentoComTipoDeContratoErrado = PeriodicConstantProcessingTime contract {
    period = 100;
    peakProcessing_per_period = 20;
};

processamentoComTipoDeValorErrado = CPU_RT_PCPT contract {
    periodMax = 40;
    peakProcessing_per_period = 0.25;
};

processamentoMinimo = CPU_RT_PCPT contract {
    period = 100;
//    peakProcessing_per_period = 20;
    peakProcessing_per_period = 70;
};

idealProfile for mpeg_player = profile {
    require processamentoIdeal;
};

minimoProfile for mpeg_player = profile {
    require processamentoMinimo;
};

profileComContratoErrado1 for mpeg_player = profile {
    require processamentoComDimensaoErrada;
};

profileComContratoErrado2 for mpeg_player = profile {
    require processamentoComTipoDeContratoErrado;
};

