package reflector;

import Pid;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import negotiator.NegotiatorInit;

import com.ibm.aglet.Aglet;
import com.ibm.aglet.AgletException;
import com.ibm.aglet.AgletProxy;
import com.ibm.aglet.InvalidAgletException;
import com.ibm.aglet.Message;
import com.ibm.aglet.MessageException;
import com.ibm.aglet.NotHandledException;
import com.ibm.aglet.event.MobilityAdapter;
import com.ibm.aglet.event.MobilityEvent;
import com.ibm.aglet.event.MobilityListener;

/*
 * Created on Jun 25, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class ReflectorAglet extends Aglet
{
	transient private AdaptStrategy adapt;
	transient ReflectorServer server;	
	ReflectorAgletDialog dialog;
	String serverAddress;
	Integer localPort;
	ReflectorConfigurator refConf;
	Abductor a;
	transient AgletProxy neg;
	
	public void onCreation(Object init)
	{
		// Adds a mobility listener
		MobilityListener mobListen = new MobilityAdapter()
		{
			public void onDispatching(MobilityEvent event)
			{
				System.out.println("[DEBUG] Disposing Negotiator Aglet");
				try
				{
					neg.dispose();
				} catch (InvalidAgletException e)
				{
					e.printStackTrace();
				}
				System.out.println("[DEBUG] Stopping threads before dispatch");
				server.stopWaiting();
				System.out.println("Antes");
				//while(refConf.isAlive())
					System.out.println("[DEBUG] Waiting for reflector configurator thread to finish");
				System.out.println("Depois");
				
				System.out.println("[MESTRADO] tempo final "+System.currentTimeMillis());
			}
			
			public void onArrival(MobilityEvent event)
			{		
				System.out.println("[DEBUG] Reflector has arrived");
				createServer(serverAddress, localPort);
				createDynConfiguration();
				createNegotiator();
			}
			
			public void onReverting(MobilityEvent event)
			{
				System.out.println("[DEBUG] Stopping threads before coming back");
				server.stopWaiting();
			}
		};
		
		addMobilityListener(mobListen);
		dialog = new ReflectorAgletDialog();
		dialog.show();
		dialog.waitResults();
		
		this.serverAddress=dialog.getServerAddress();
		this.localPort=dialog.getLocalPort();
		System.out.println(serverAddress);
		createServer(serverAddress, localPort);
		
		// Creates thread for dynamic configuration
		createDynConfiguration();
		
		// Creates aglet for QoS negotiation
		createNegotiator();
		//NegotiatorInit negInit=new NegotiatorInit("QOS_dsrt.qml", pid.getPid()); 
		
		// Add classes that need to be transported
		a = Abductor.instance();
		try
		{
			a.addObject(Class.forName("reflector.AdaptStrategy"));
			a.addObject(Class.forName("reflector.AdaptMigrate"));
			a.addObject(Class.forName("reflector.AdaptClone"));
			a.addObject(Class.forName("reflector.ReflectorServer"));
			a.addObject(Class.forName("reflector.ReflectorConnectionHandler"));
			a.addObject(Class.forName("reflector.BandwidthDialog"));
			a.addObject(Class.forName("reflector.CountingPrintWriter"));
			a.addObject(Class.forName("reflector.RoamingStrategy"));
			a.addObject(Class.forName("reflector.RoamingStrategyParallel"));
			a.addObject(Class.forName("reflector.RoamingStrategyLinear"));
			a.addObject(Class.forName("Pid"));
			a.addObject(Class.forName("negotiator.NegotiatorInit"));
			a.addObject(Class.forName("negotiator.Negotiator"));
			a.addObject(Class.forName("negotiator.NegotiationAgentInit"));
			a.addObject(Class.forName("negotiator.NegotiationResult"));
		}
		catch (ClassNotFoundException e)
		{
			e.printStackTrace();
		}
	}
	
	public void createNegotiator()
	{
		Pid pid=new Pid();
		NegotiatorInit negInit = null;
		
		try
		{
			negInit = new NegotiatorInit(new URL("http://durden/QOS_dsrt.qml"), pid.getPid(), RoamingStrategy.PARALLEL_ROAMING,  this.getProxy()); 

			neg = getAgletContext().createAglet(getCodeBase(), "negotiator.Negotiator", negInit);
	
		} catch (MalformedURLException e1)
		{
			e1.printStackTrace();
		} catch (IOException e)
		{
			e.printStackTrace();
		} catch (AgletException e)
		{
			e.printStackTrace();
		} catch (ClassNotFoundException e)
		{
			e.printStackTrace();
		} catch (InstantiationException e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void createServer(String serverAddress, Integer localPort)
	{	
		try
		{
			server = new ReflectorServer(serverAddress, localPort);
		} catch (IOException e)
		{
			System.out.println("[ERROR] Port "+localPort.intValue()+" already open by another application");
			e.printStackTrace();
		}
	}
	
	private void createDynConfiguration()
	{
		try
		{
			this.refConf = new ReflectorConfigurator(this, this.getClass().getDeclaredMethod("adaptationStrategy", null));
		}
		catch (SecurityException e)
		{
			e.printStackTrace();
		}
		catch (NoSuchMethodException e)
		{
			e.printStackTrace();
		}
	}
	
	public void run()
	{
		Pid pid=new Pid();
				
		System.out.println(pid.getPid());
		server.start();
		refConf.start();
	}
	
	public void onDisposing()
	{
		System.out.println("[DEBUG] Disposing Reflector Aglet");
		server.stopWaiting();
	} 
	
	public void adaptationStrategy()
	{	
		AgletProxy reflectorProxy = getAgletContext().getAgletProxy(getAgletID());		
		adapt = new AdaptMigrate(reflectorProxy, neg);
		//adapt = new AdaptClone(reflectorProxy, neg);
		//adapt.negotiate();
		
		try
		{
			neg.sendMessage(new Message("startNegotiation"));
		} catch (InvalidAgletException e)
		{
			e.printStackTrace();
		} catch (NotHandledException e)
		{
			e.printStackTrace();
		} catch (MessageException e)
		{
			e.printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.aglet.Aglet#handleMessage(com.ibm.aglet.Message)
	 */
	public boolean handleMessage(Message msg)
	{
		if(msg.sameKind("negotiationResults"))
		{
			Vector res = (Vector)msg.getArg();
			adapt.setDestinations(res);
			adapt.start();
			return(true);
		}
		
		return(false);
	}
}