/*
 * Created on Jun 30, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package reflector;

import java.io.IOException;
import java.io.InterruptedIOException;
import java.io.Serializable;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.Iterator;
import java.util.Vector;

/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class ReflectorServer extends Thread implements Serializable
{
	transient ServerSocket server;
	transient boolean keepWaiting;
	transient Vector connectionHandlers;
	
	ReflectorConnectionHandler teletransporter;
	String serverAddress;
	Integer localPort;
	
	public ReflectorServer(String serverAddress, Integer localPort) throws IOException
	{
		this.serverAddress=serverAddress;
		this.localPort=localPort;
		server = new ServerSocket(localPort.intValue());		
		keepWaiting = true;
		this.connectionHandlers=new Vector();
		
		this.teletransporter = new ReflectorConnectionHandler(serverAddress, null);
	}
	
	public ReflectorServer()
	{}
		
	public void run()
	{
		// Checks each second if the thread was stopped
		try
		{
			server.setSoTimeout(1000);
		} catch (SocketException e1)
		{
			e1.printStackTrace();
		}
		
		Socket connection = null;
		while(keepWaiting)
		{
			
				try
				{
					connection = server.accept();
					ReflectorConnectionHandler handler = 
					new ReflectorConnectionHandler(serverAddress, connection);
					this.connectionHandlers.add(handler);
					handler.start();
				} catch (InterruptedIOException e)
				{
					//e.printStackTrace();
					//System.out.println("Keep waiting...");
				} 
				catch (IOException e)
				{
					//e.printStackTrace();
					//System.out.println("[DEBUG] " + e.toString());
				}
		}
	}
	
	public void stopWaiting()
	{
		keepWaiting=false;
		try
		{
			this.server.close();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
		stopHandlers();
	}

	/**
	 * 
	 */
	private void stopHandlers()
	{
		Iterator handlers=this.connectionHandlers.iterator();
		while(handlers.hasNext())
		{
			ReflectorConnectionHandler h=(ReflectorConnectionHandler)handlers.next();
			h.stopSending();			
		}
	}
}
