/*
 * Created on Jun 30, 2003
 *
 * Created by Roberto S. Cardoso <speicys@ime.usp.br>
 */
package soundtest;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.InterruptedIOException;
import java.io.PrintWriter;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.URL;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

import org.tritonus.share.sampled.AudioFileTypes;
import org.tritonus.share.sampled.Encodings;


/**
 * @author Roberto S. Cardoso
 *
 * 
 */
public class MixerHandler extends Thread implements Serializable
{
	transient private boolean keepSending;
	transient Socket connection;
	transient URL streamingServerAddr1;
	transient URL streamingServerAddr2;
	transient Socket streamingServer1;
	transient Socket streamingServer2;
	transient BufferedReader input1;
	transient BufferedReader input2;
	
	InputStream is1;
	InputStream is2;
	
	transient PrintWriter outputToServer1;
	transient PrintWriter outputToServer2;
		
	PrintWriter codeWriter;
	PrintWriter outputToClient;
	BufferedOutputStream osClient;
	
	String serverAddress1, serverAddress2;
	
	private static final AudioFormat.Encoding	MPEG1L3 = Encodings.getEncoding("MPEG1L3");
	private static final AudioFileFormat.Type	MP3 = AudioFileTypes.getType("MP3", "mp3");
	private static final int HTTP_HEADER_SIZE = 205;
	
	public MixerHandler(String serverAddress1, String serverAddress2, Socket connection)
	{
		this.connection=connection;
		this.keepSending=true;
		
		this.serverAddress1=serverAddress1;
		this.serverAddress2=serverAddress2;
		//this.codeWriter = new PrintWriter(null);
	}
	
	public static void main(String []args)
	{
		String serverAddress1="http://localhost:8000/mp3";
		String serverAddress2="http://localhost:8000/lib";
		
		int localPort=7733;
		ServerSocket server = null;
		try
		{
			server = new ServerSocket(localPort);
		} catch (IOException e2)
		{
			e2.printStackTrace();
		}		
		boolean keepWaiting = true;
			
		// Checks each second if the thread was stopped
		try
		{
			server.setSoTimeout(1000);
		} catch (SocketException e1)
		{
			e1.printStackTrace();
		}
		
		Socket connection = null;
		while(keepWaiting)
		{
			try
			{
				connection = server.accept();
				MixerHandler handler = new MixerHandler(serverAddress1, serverAddress2, connection);
				handler.start();
			} catch (InterruptedIOException e)
			{
				//e.printStackTrace();
				//System.out.println("Keep waiting...");
			} 
			catch (IOException e)
			{
				e.printStackTrace();
				System.out.println("[DEBUG] " + e.toString());
			}
		}
	}
	
	public void run()
	{
		try
		{
			streamingServerAddr1 = new URL(serverAddress1);
			streamingServerAddr2 = new URL(serverAddress2);

			String host1=streamingServerAddr1.getHost();
			String host2=streamingServerAddr2.getHost();
			int port1=streamingServerAddr1.getPort();
			if(port1==-1)
				port1=80;
			int port2=streamingServerAddr2.getPort();
			if(port2==-1)
				port2=80;

			System.out.println("[DEBUG] Addr -> "+host1);
			System.out.println("[DEBUG] Port -> "+port1);
			System.out.println("[DEBUG] File -> "+streamingServerAddr1.getFile());
		
			System.out.println();
			System.out.println("[DEBUG] Addr -> "+host2);
			System.out.println("[DEBUG] Port -> "+port2);
			System.out.println("[DEBUG] File -> "+streamingServerAddr2.getFile());
										
			streamingServer1=new Socket(host1, port1);
			streamingServer2=new Socket(host1, port1);
			
			outputToClient = new PrintWriter(new PrintWriter(connection.getOutputStream(), true));
			osClient=new BufferedOutputStream(connection.getOutputStream());
			
			input1 = new BufferedReader(new InputStreamReader(streamingServer1.getInputStream()));
			input2= new BufferedReader(new InputStreamReader(streamingServer2.getInputStream()));
			
			is1 = streamingServer1.getInputStream();
			is2 = streamingServer2.getInputStream();
			
			outputToServer1 = new PrintWriter(streamingServer1.getOutputStream(), true);
			outputToServer2 = new PrintWriter(streamingServer2.getOutputStream(), true);
			
			// Begins http connection
			System.out.println("GET "+streamingServerAddr1.getFile()+" HTTP/1.0\r \n");
			System.out.println("GET "+streamingServerAddr2.getFile()+" HTTP/1.0\r \n");
			
			outputToServer1.println("GET "+streamingServerAddr1.getFile()+" HTTP/1.0\r \n");
			outputToServer2.println("GET "+streamingServerAddr2.getFile()+" HTTP/1.0\r \n");
			
		} catch (MalformedURLException e)
		{
			e.printStackTrace();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
	
		int BUFFER_SIZE = 10 * 1024;
		char[] inputChar1 = new char[BUFFER_SIZE];
		char[] inputChar2 = new char[BUFFER_SIZE];
		
		byte[] inputByte1 = new byte[BUFFER_SIZE];
		byte[] inputByte2 = new byte[BUFFER_SIZE];
		
		byte[] readByte1 = new byte[BUFFER_SIZE];
		byte[] readByte2 = new byte[BUFFER_SIZE];
		
		byte[] mixedByte = new byte[BUFFER_SIZE];
		byte[] mixedEncByte = new byte[BUFFER_SIZE];
		
		ByteArrayInputStream mixedRaw;
	
		try
		{
			int i=0;
			int res1,res2;
			
			//SourceDataLine line;

			do
			{
				 //res1=input1.read(inputChar1, 0, inputChar1.length);
				//res2=input2.read(inputChar2, 0, inputChar2.length);
				//res1=input1.read(inputChar1);
				//res2=input2.read(inputChar2);
				
				res1 = is1.read(inputByte1, 0, HTTP_HEADER_SIZE);
				res2 = is2.read(inputByte2);
								
				AudioInputStream ais1 = null;
				AudioInputStream ais2 = null;				
				
				try
				{
					ais1=AudioSystem.getAudioInputStream(new URL("http://localhost:8000/mp3"));
					ais2=AudioSystem.getAudioInputStream(new URL("http://localhost:8000/lib"));
				} 
				catch (UnsupportedAudioFileException e2)
				{
					e2.printStackTrace();
				}
				
				AudioFormat af1=ais1.getFormat();
				AudioFormat af2=ais2.getFormat();
				DataLine.Info info1 = new DataLine.Info(SourceDataLine.class, af1, 40960);
				DataLine.Info info2 = new DataLine.Info(SourceDataLine.class, af2, 40960);
												
				boolean	bIsSupportedDirectly = AudioSystem.isLineSupported(info1);
				if (!bIsSupportedDirectly)
				{
					AudioFormat sourceFormat = af1;
					AudioFormat targetFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, sourceFormat.getSampleRate(),
								16, sourceFormat.getChannels(), sourceFormat.getChannels() * (16/8), sourceFormat.getSampleRate(),
								false);
					System.out.println("AudioPlayer.main(): source format: " + sourceFormat);
					System.out.println("AudioPlayer.main(): target format: " + targetFormat);
					ais1 = AudioSystem.getAudioInputStream(targetFormat, ais1);
					af1 = ais1.getFormat();
					System.out.println("AudioPlayer.main(): converted AIS: " + ais1);
					System.out.println("AudioPlayer.main(): converted format: " + af1);
				}

				boolean	bIsSupportedDirectly2 = AudioSystem.isLineSupported(info2);
				if (!bIsSupportedDirectly2)
				{
					AudioFormat sourceFormat = af2;
					AudioFormat targetFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, sourceFormat.getSampleRate(),
								16, sourceFormat.getChannels(), sourceFormat.getChannels() * (16/8), sourceFormat.getSampleRate(),
								false);
					System.out.println("AudioPlayer.main(): source format: " + sourceFormat);
					System.out.println("AudioPlayer.main(): target format: " + targetFormat);
					ais2 = AudioSystem.getAudioInputStream(targetFormat, ais2);
					af2 = ais2.getFormat();
					System.out.println("AudioPlayer.main(): converted AIS: " + ais2);
					System.out.println("AudioPlayer.main(): converted format: " + af2);
				}

				/*line = getSourceDataLine(null, af1, 40960);
				if (line == null)
				{
					System.out.println("AudioPlayer: cannot get SourceDataLine for format " + af1);
					System.exit(1);
				}

				System.out.println("AudioPlayer.main(): line: " + line);
				System.out.println("AudioPlayer.main(): line format: " + line.getFormat());
				
				line.start();*/
				
				osClient.write(inputByte1, 0, HTTP_HEADER_SIZE);
				
				int nBytesRead=0;
				int nBytesRead1=0;
				int nBytesRead2=0;
				
				AudioInputStream aisEnc = null;
				System.setProperty("tritonus.lame.quality", "lowest");
				System.setProperty("tritonus.lame.vbr", "false");
				System.setProperty("tritonus.lame.bitrate", "8");
							
				while (nBytesRead != -1)
				{
					try
					{
						nBytesRead1 = ais1.read(readByte1, 0, readByte1.length);
						nBytesRead2 = ais2.read(readByte2, 0, readByte2.length);
						nBytesRead = Math.max(nBytesRead1, nBytesRead2);
						
						for(int x=0; x<readByte1.length; x++)
						{
							mixedByte[x]=(byte) ((readByte1[x]+readByte2[x])/2);
						}
					}
					catch (IOException e)
					{
						e.printStackTrace();
					}
					if (nBytesRead >= 0)
					{
						AudioInputStream aisMixedRaw = null;
						mixedRaw = new ByteArrayInputStream(mixedByte);
				
						int sizeInFrames = nBytesRead/af1.getFrameSize();
						aisMixedRaw = new AudioInputStream(mixedRaw, af1, AudioSystem.NOT_SPECIFIED);

						/*if(aisEnc==null)
						{						
							aisEnc = encode(aisMixedRaw, MPEG1L3);
							System.out.println("Gastou tempo...");
						}*/
						
						aisEnc = encode(aisMixedRaw, MPEG1L3);
						aisEnc.read(mixedEncByte, 0, nBytesRead);
						//aisMixedRaw.read(mixedEncByte, 0, nBytesRead);
						//line.write(mixedByte, 0, nBytesRead);
												
						osClient.write(mixedEncByte, 0, nBytesRead);
												
						//AudioSystem.write(aisEnc, MP3, osClient);
						//int	nBytesWritten = line.write(mixedByte, 0, nBytesRead);
						//osClient.write(mixedByte);
					}
				}
								
			}while(res1!=-1 ||  res2!=-1);
						
			/*line.drain();
			line.close();*/
			
		} catch (IOException e1)
		{
			e1.printStackTrace();
		}
	}
	
	public AudioInputStream encode(AudioInputStream ais, AudioFormat.Encoding targetEncoding)
	{
		AudioFormat sourceFormat=ais.getFormat();
		//System.out.println("Input format: " + sourceFormat);
		
		// construct a converted stream
		AudioInputStream targetStream = null;
		if (!AudioSystem.isConversionSupported(targetEncoding, sourceFormat)) 
		{
			System.out.println("direct conversion not supported");
			AudioFormat intermediateFormat = new AudioFormat(
												  AudioFormat.Encoding.PCM_SIGNED,
												  sourceFormat.getSampleRate(),
												  16,
												  sourceFormat.getChannels(),
												  2 * sourceFormat.getChannels(), // frameSize
												  sourceFormat.getSampleRate(),
												  false);
			if (AudioSystem.isConversionSupported(intermediateFormat, sourceFormat)) 
			{
				// intermediate conversion is supported
				System.out.println("intermediate conversion supported");
				ais = AudioSystem.getAudioInputStream(intermediateFormat, ais);
			}
		}
		
		targetStream = AudioSystem.getAudioInputStream(targetEncoding, ais);
		if (targetStream == null)
		{
			System.out.println("conversion not supported");
		}
		
		/*System.out.println("Got converted AudioInputStream: " + targetStream.getClass().getName());
		System.out.println("Output format: " + targetStream.getFormat());*/
		 
		 byte[] bTst = new byte[8];
		 try
		{
			targetStream.read(bTst, 0, bTst.length);
		} catch (IOException e)
		{
			e.printStackTrace();
		}
		 
		return targetStream;
	}
	
	public void stopSending()
	{
		this.keepSending=false;
	}
	
	private static SourceDataLine getSourceDataLine(String strMixerName,
									AudioFormat audioFormat,
									int nBufferSize)
		{
			/*
			 *	Asking for a line is a rather tricky thing.
			 *	We have to construct an Info object that specifies
			 *	the desired properties for the line.
			 *	First, we have to say which kind of line we want. The
			 *	possibilities are: SourceDataLine (for playback), Clip
			 *	(for repeated playback)	and TargetDataLine (for
			 *	 recording).
			 *	Here, we want to do normal playback, so we ask for
			 *	a SourceDataLine.
			 *	Then, we have to pass an AudioFormat object, so that
			 *	the Line knows which format the data passed to it
			 *	will have.
			 *	Furthermore, we can give Java Sound a hint about how
			 *	big the internal buffer for the line should be. This
			 *	isn't used here, signaling that we
			 *	don't care about the exact size. Java Sound will use
			 *	some default value for the buffer size.
			 */
			SourceDataLine	line = null;
			DataLine.Info	info = new DataLine.Info(SourceDataLine.class,
								 audioFormat, nBufferSize);
			try
			{
				if (strMixerName != null)
				{
					Mixer.Info	mixerInfo = AudioCommon.getMixerInfo(strMixerName);
					if (mixerInfo == null)
					{
						System.out.println("AudioPlayer: mixer not found: " + strMixerName);
						System.exit(1);
					}
					Mixer	mixer = AudioSystem.getMixer(mixerInfo);
					line = (SourceDataLine) mixer.getLine(info);
				}
				else
				{
					line = (SourceDataLine) AudioSystem.getLine(info);
				}

				/*
				 *	The line is there, but it is not yet ready to
				 *	receive audio data. We have to open the line.
				 */
				line.open(audioFormat, nBufferSize);
			}
			catch (LineUnavailableException e)
			{
				e.printStackTrace();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			return line;
		}
}
