/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (C) 2001 Gerwin Klein <lsf@jflex.de>                          *
 * All rights reserved.                                                    *
 *                                                                         *
 * This is a modified version of the example from                          *
 *   http://www.lincom-asg.com/~rjamison/byacc/                            *
 *                                                                         *
 * Thanks to Larry Bell and Bob Jamison for suggestions and comments.      *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License. See the file      *
 * COPYRIGHT for more information.                                         *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                 *
 *                                                                         *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

%{
  import java.io.*;
  import java.util.*;
%}
      

%token NL          /* newline  */

/* keywords */
%token CONTRACT
%token DECREASING
%token FOR
%token FROM
%token INCREASING
%token NUMERIC
%token PERCENTILE
%token PROFILE
%token REQUIRE
%token TYPE

/* values */
%token NUM
%token STRING

%%

especification: 
              | especification contract_type
              | especification contract
              | especification profile
              ;

contract_type: type_definition "{" dimension "}" ";" { /*System.out.println(contract_types);*/ }
             ;

type_definition: TYPE STRING "=" CONTRACT { System.out.println("[Y] Contract Type name: " + $2.sval);
                                            Vector dimensions = new Vector();
                                            contract_types.put($2.sval, dimensions); 
                                            parsingEntity=new String($2.sval);}
               ;
  
dimension: STRING ":" dim_type ";" { ((Vector)contract_types.get(parsingEntity)).add($1.sval);
                                     System.out.println("[Y] Dim name: " + $1.sval); }
         | dimension STRING ":" dim_type ";" { ((Vector)contract_types.get(parsingEntity)).add($2.sval);
                                               System.out.println("[Y] Dim name: " + $2.sval); }
         ;

dim_type: order NUMERIC unit
        ;

unit: 
    | STRING
    ;

order: INCREASING
     | DECREASING
     ;

contract: contract_definition "{" values "}" ";"
        ;

contract_definition: STRING "=" STRING CONTRACT { if(contract_types.containsKey($3.sval))
                                                  {    
                                                      System.out.println("[Y] Contract " + $1.sval + " of Type " + $3.sval); 
						      parsingEntity=new String($3.sval);
						      contracts.put($1.sval, $3.sval);
						      //dimension_values.put($1.sval, new Hashtable());
						  }
                                                  else
                                                  {
                                                      yyerror("Contract type " + $3.sval + " was not defined.");
						      //exit(0);
						  } 
                                                }
                   ;

values: STRING operand NUM unit ";" { if(!((Vector)contract_types.get(parsingEntity)).contains($1.sval))
                                      {
                                           yyerror("Dimension " + $1.sval + " not defined for Contract type " + parsingEntity + ".");
					   contracts.remove(parsingEntity);
					   //dimension_values.remove(parsingEntity);
				      }
                                      else 
				      {/*
					Hashtable dim_val = (Hashtable)dimension_values.get(parsingEntity);
					if(dim_val==null)
					  System.out.println("Null Pointer!!!!!!!!");
					
					  dim_val.put($1.sval,$3.sval);*/
				      }				  
                                    }
      | values STRING operand NUM unit ";" { if(!((Vector)contract_types.get(parsingEntity)).contains($2.sval))
                                             {
                                                 yyerror("Dimension " + $2.sval + " not defined for Contract type " + parsingEntity + ".");
						 contracts.remove(parsingEntity);
						 //dimension_values.remove(parsingEntity);
					     }
                                             else
					     {/*
					       Hashtable dim_val = (Hashtable)dimension_values.get(parsingEntity);
					       dim_val.put($2.sval,$4.sval);*/
					     }
                                           }
      ;

operand: "="
       | "<"
       | ">"
       | ">" "="
       | "<" "="
       ;

profile: profile_definition "{" requirements "}" ";" { /*System.out.println(profiles);*/ }
       ;

profile_definition: STRING FOR STRING "=" PROFILE { System.out.println("[Y] Profile " + $1.sval + " for application " + $3.sval); 
                                                    parsingEntity=new String($1.sval);
						    applications.put($1.sval, $3.sval);			    
                                                  }
                  ;

requirements: REQUIRE STRING ";" { if(!contracts.containsKey($2.sval))
                                   {
				     yyerror("Contract " + $2.sval + " not defined.");
				     applications.remove(parsingEntity);				     
				   } 
                                   else
				   {
				     profiles.put(parsingEntity, $2.sval);
				   }			       
                                 }  
            ;
 
%%

  private Yylex lexer;


  private int yylex () {
    int yyl_return = -1;
    try {
      yylval = new ParserVal(0);
      yyl_return = lexer.yylex();
    }
    catch (IOException e) {
      System.err.println("IO error :"+e);
    }
    return yyl_return;
  }


  public void yyerror (String error) {
    System.err.println ("Error: " + error);
  }


  public Parser(Reader r) {
    lexer = new Yylex(r, this);
  }


  static boolean interactive;
  // Hash that associates a contract type to its dimensions
  static Hashtable contract_types;
  // Name of the entity being parsed.
  static String parsingEntity;
  // Hashtable that contains the contracts defined
  static Hashtable contracts;
  // Hashtable that contains each contract's values of the dimensions
  static Hashtable dimension_values;
  // Vector that contains the DSRT contracts especifications
  static Vector dsrt_contracts;
  // Hashtable containing the profiles defined for each application
  static Hashtable profiles;
  // Hashtable containing the applications and the profiles defined for them
  static Hashtable applications;


  public static void main(String args[]) throws IOException {
    System.out.println("BYACC/Java with JFlex QML->DSRT Compiler");
    
    int yydebug=1; 

    Parser yyparser;
    
    contract_types = new Hashtable();
    contracts = new Hashtable();
    dsrt_contracts = new Vector();
    profiles = new Hashtable();
    applications = new Hashtable();
    dimension_values = new Hashtable();
    
            
    if ( args.length > 0 ) {
      // parse a file
      yyparser = new Parser(new FileReader(args[0]));
    }
    else {
      // interactive mode
      System.out.println("[Quit with CTRL-D]");
      System.out.print("Expression: ");
      interactive = true;
	    yyparser = new Parser(new InputStreamReader(System.in));
    }

    yyparser.yyparse();

    // After parsing the file, we can build the strings for reservation
    Vector reservation_strings = new Vector();

    for(Enumeration e = applications.keys(); e.hasMoreElements();) {
      String profile = (String)e.nextElement();
      String command_line = new String();
      command_line = command_line.concat((String)applications.get(profile));
      command_line = command_line.concat(" -reserve ");
      
      String contract = (String)profiles.get(profile);
      String contract_type = (String)contracts.get(contract);
      command_line = command_line.concat(contract_type);
      command_line = command_line.concat(" ");
      
      /*Vector dims = (Vector)contract_types.get(contract_type);
      Hashtable dim_values = (Hashtable)dimension_values.get(contract);
      for(Enumeration e_dim = dims.elements(); e_dim.hasMoreElements();)
      {
	command_line.concat((String)dim_values.get(e_dim.nextElement()));
	command_line.concat(" ");
	}*/
      
      reservation_strings.add(command_line);
     }
    
    System.out.println(reservation_strings);
        
    if (interactive) {
      System.out.println();
      System.out.println("Have a nice day");
    }
  }
